﻿SET NOCOUNT ON

--the script checks the database format
--and updates it to the latest version if necessary
--run it against the Helpdesk's database to upgrade it!
--PS. most recent changes are at the bottom... top-to-bottom order.


--###########################################################################

--fileAttachments table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[hdFileAttachments]') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdFileAttachments](
		[FileID] [int] IDENTITY(1,1) PRIMARY KEY NOT NULL,
		[IssueID] [int] NOT NULL,
		[UserID] [int] NOT NULL,
		[FileName] [varchar](255) NOT NULL,
		[FileData] [image] NULL
	)
END
GO

--allowUserRegistration column
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AllowUserRegistration')
begin
	  ALTER TABLE hdSettings ADD AllowUserRegistration bit NOT NULL DEFAULT 1
end

--###########add MailChecker columns
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerNewIssuesCategoryID')
begin
	  ALTER TABLE hdSettings ADD MailCheckerNewIssuesCategoryID int NOT NULL DEFAULT (1)
end
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerAllowUnregisteredUsers')
begin
	  ALTER TABLE hdSettings ADD MailCheckerAllowUnregisteredUsers bit NOT NULL DEFAULT (0)
end
--#############

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='Priority')
begin
	  ALTER TABLE hdIssues ADD Priority smallint NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SMTPPort')
begin
	  ALTER TABLE hdSettings ADD SMTPPort int NOT NULL DEFAULT 25
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SMTPUseSSL')
begin
	  ALTER TABLE hdSettings ADD SMTPUseSSL bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='RestrictIssueDeletion')
begin
	  ALTER TABLE hdSettings ADD RestrictIssueDeletion bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='Lang')
begin
	  ALTER TABLE hdSettings ADD Lang [varchar] (10) NOT NULL DEFAULT ('en-US')
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='FirstName')
begin
	  ALTER TABLE hdUsers ADD FirstName varchar(255) NOT NULL DEFAULT ('')
end
if not exists (select * from syscolumns where id=object_id('hdUsers') and name='LastName')
begin
	  ALTER TABLE hdUsers ADD LastName varchar(255) NOT NULL DEFAULT ('')
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='RestrictIssueClosing')
begin
	  ALTER TABLE hdSettings ADD RestrictIssueClosing bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Notes')
begin
	  ALTER TABLE hdUsers ADD Notes varchar(4000) NOT NULL DEFAULT ('')
end

--Sections table		  
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[hdSections]') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdSections] (
	[SectionID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[Name] [varchar] (50) NOT NULL
	)
END

if not exists (select * from syscolumns where id=object_id('hdCategories') and name='SectionID')
begin
	  ALTER TABLE hdCategories ADD SectionID int NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='NewIssuesDefaultCategoryID')
begin
	  ALTER TABLE hdSettings ADD NewIssuesDefaultCategoryID int NOT NULL DEFAULT 1
end
GO

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Location')
begin
	  ALTER TABLE hdUsers ADD Location varchar(1000) NULL
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Phone')
begin
	  ALTER TABLE hdUsers ADD Phone varchar(50) NULL
end

if not exists (select * from syscolumns where id=object_id('hdComments') and name='ForTechsOnly')
begin
	  ALTER TABLE hdComments ADD ForTechsOnly bit NOT NULL DEFAULT 0
end


/*#####!!!multiinstance changes start #################*/

--if hdInstances - rename
IF EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdInstances') AND type in (N'U')) AND NOT EXISTS(SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'Instances') AND type in (N'U'))
begin
	exec sp_rename 'hdInstances', 'Instances'
end

--InstanceID column
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='InstanceID')
begin
	  ALTER TABLE hdSettings ADD InstanceID int NOT NULL DEFAULT 0
	  ALTER TABLE [hdCategories] ADD InstanceID int NOT NULL DEFAULT 0
	  ALTER TABLE [hdSections] ADD InstanceID int NOT NULL DEFAULT 0
	  ALTER TABLE [hdIssues] ADD InstanceID int NOT NULL DEFAULT 0
	  ALTER TABLE [hdUsers] ADD InstanceID int NOT NULL DEFAULT 0
end
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[Instances]') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[Instances] (
		[InstanceID] int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[Name] [varchar] (100) NOT NULL,
		[CustomerID] int NOT NULL,
		ValidTill datetime NOT NULL
	)
END
GO

--drop unique user name constraint
if exists (select * from sysobjects where name = 'IX_hdUsers_UniqueName')
begin
	alter table hdUsers drop constraint IX_hdUsers_UniqueName
	
	--add username/instanceid uniqeness
	if not exists (select * from sysobjects where name = 'IX_hdUsers_UniqueNameInstance')
	begin
		ALTER TABLE [dbo].[hdUsers] WITH NOCHECK ADD 
		CONSTRAINT [IX_hdUsers_UniqueNameInstance] UNIQUE  NONCLUSTERED 
		(
			[username],
			InstanceID
		)
	end
end
/*##################!!!!!!!!!!end of multiinstance code*/
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='UseDefaultSmtp')
begin
	  ALTER TABLE hdSettings ADD UseDefaultSmtp bit NOT NULL DEFAULT 0
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AllowAnonymousKBAccess')
begin
	  ALTER TABLE hdSettings ADD AllowAnonymousKBAccess bit not null default 1
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AllowGuestSubmission')
begin
	  ALTER TABLE hdSettings ADD AllowGuestSubmission bit not null default 1
end
GO

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='SendEmail')
begin
	  ALTER TABLE hdUsers ADD SendEmail bit not null default 1
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='HeaderBGColor')
begin
	  ALTER TABLE hdSettings ADD HeaderBGColor char(6) NOT NULL DEFAULT('000000')
	  ALTER TABLE hdSettings ADD MenuBarBGColor char(6) NOT NULL DEFAULT('FF7700')
	  ALTER TABLE hdSettings ADD LogoImage [image] NULL
end
GO
	
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SeeEveryonesIssues')
begin
	  ALTER TABLE hdSettings ADD SeeEveryonesIssues bit NOT NULL DEFAULT 0
end
GO

--enlarge the password field so MD5-string fits there
if exists(select name from syscolumns where id=object_id('hdUsers') and name='Password' and length<50)
begin
	alter table hdUsers alter column Password [varchar](50) NOT NULL
end

--dropping IX_hdUsers_UniqueEmailInstance constraint - not needed.
if exists (select * from sysobjects where name = 'IX_hdUsers_UniqueEmailInstance')
begin
	alter table hdUsers drop constraint IX_hdUsers_UniqueEmailInstance
end
GO

--converting Body and Subject and Status from varchar to Nvarchar
if exists (select id from syscolumns where id=object_id('hdIssues') and name='Subject' and type_name(xtype)<>'nvarchar')
	alter table hdIssues alter column Subject nvarchar(1000) NOT NULL
if exists (select id from syscolumns where id=object_id('hdStatus') and name='Name' and type_name(xtype)<>'nvarchar')
	alter table hdStatus alter column [Name] nvarchar (50) NOT NULL
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SendTechniciansEmailOnNewIssue')
begin
	ALTER TABLE hdSettings ADD SendTechniciansEmailOnNewIssue bit NOT NULL DEFAULT 1
end

--adding "Custom Fields" functionality
if not exists (select * from INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='hdCustomFields')
begin
	CREATE TABLE [dbo].[hdCustomFields] (
		[FieldID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] [int] NOT NULL,
		[Type] [int] NOT NULL,
		[FieldName] nvarchar(100) NOT NULL
	)
	CREATE TABLE [dbo].[hdCustomFieldValues] (
		[IssueID] [int] NOT NULL,
		[FieldID] [int] NOT NULL,
		[Value] sql_variant NOT NULL
	)
	ALTER TABLE [dbo].[hdCustomFieldValues] WITH NOCHECK ADD 
		CONSTRAINT [PK_hdCustomFieldValues]PRIMARY KEY CLUSTERED ([IssueID],[FieldID])
	CREATE TABLE [dbo].[hdCustomFieldOptions] (
		[OptionID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[FieldID] [int] NOT NULL,
		[OptionValue] nvarchar(500) NOT NULL
	)
end
GO


--enlarging username and email columns
if exists(select name from syscolumns where id=object_id('hdUsers') and name='UserName' and length<100)
	alter table hdUsers alter column UserName varchar(100) NOT NULL
if exists(select name from syscolumns where id=object_id('hdUsers') and name='Email' and length<100)
	alter table hdUsers alter column Email varchar(100) NOT NULL

--adding index to hdComments (IssueID
if not exists (select name FROM sysindexes WHERE name = 'IX_hdComments_IssueID') begin
	CREATE INDEX IX_hdComments_IssueID ON hdComments (IssueID)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='ReplyToAddress')
begin
	ALTER TABLE hdSettings ADD ReplyToAddress varchar(50) NULL
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='StartDate')
begin
	ALTER TABLE hdIssues ADD StartDate datetime NULL
end

--hdIssueSubscribers table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[hdIssueSubscribers]') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdIssueSubscribers] (
		[IssueID] [int] NOT NULL,
		[UserID] [int] NOT NULL
	)
END
GO
IF NOT EXISTS (SELECT * FROM sysforeignkeys WHERE constid = OBJECT_ID(N'[dbo].[FK_hdIssueSubscribers_hdIssues]'))
ALTER TABLE hdIssueSubscribers WITH CHECK ADD CONSTRAINT [FK_hdIssueSubscribers_hdIssues] FOREIGN KEY([IssueID])
REFERENCES hdIssues (IssueID)
GO
ALTER TABLE hdIssueSubscribers CHECK CONSTRAINT [FK_hdIssueSubscribers_hdIssues]
GO
IF NOT EXISTS (SELECT * FROM sysforeignkeys WHERE constid = OBJECT_ID(N'[dbo].[FK_hdIssueSubscribers_hdUsers]'))
ALTER TABLE hdIssueSubscribers WITH CHECK ADD CONSTRAINT [FK_hdIssueSubscribers_hdUsers] FOREIGN KEY([UserID])
REFERENCES hdUsers (UserID)
GO
ALTER TABLE hdIssueSubscribers CHECK CONSTRAINT [FK_hdIssueSubscribers_hdUsers]
GO
--hdIssueSubscribers table finished

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='LastUpdated')
begin
	  ALTER TABLE hdIssues ADD LastUpdated datetime NOT NULL DEFAULT(getdate())
end

if not exists (select * from syscolumns where id=object_id('hdStatus') and name='ButtonCaption')
begin
	  ALTER TABLE hdStatus ADD ButtonCaption nvarchar(50) NULL
end
GO

if not exists (select * from syscolumns where id=object_id('hdStatus') and name='InstanceID')
begin
	  ALTER TABLE hdStatus ADD InstanceID int NULL
end

--converting from nvarchar(4000) to Nvarchar(MAX)
if exists(select name from syscolumns where id=object_id('hdIssues') and name='Body' and length=4000)
	alter table hdIssues alter column Body nvarchar(MAX) NOT NULL
if exists(select name from syscolumns where id=object_id('hdComments') and name='Body' and length=4000)
	alter table hdComments alter column Body nvarchar(MAX) NOT NULL
GO

if not exists (select * from syscolumns where id=object_id('hdCustomFields') and name='ForTechsOnly')
begin
	  ALTER TABLE hdCustomFields ADD ForTechsOnly bit NOT NULL DEFAULT (0)
end

--asset management tables
if not exists (select * from INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='hdAssets')
begin
	CREATE TABLE [dbo].[hdAssets] (
		ItemID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		ModelName nvarchar(1000) NOT NULL,
		TypeID int NOT NULL,
		ManufacturerID int NOT NULL,
		Quantity int NOT NULL DEFAULT(0),
		SupplierID int NOT NULL
	)
	CREATE TABLE [dbo].[hdAssetManufacturers] (
		ManufacturerID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		[Name] nvarchar(1000) NOT NULL
	)
	CREATE TABLE [dbo].[hdAssetSuppliers] (
		SupplierID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		[Name] nvarchar(1000) NOT NULL
	)
	CREATE TABLE [dbo].[hdAssetTypes] (
		TypeID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		[Name] nvarchar(1000) NOT NULL
	)
	CREATE TABLE [dbo].[hdAssetUsers] (
		ItemID int NOT NULL,
		UserID int NOT NULL
	)
	ALTER TABLE [dbo].[hdAssetUsers] WITH NOCHECK ADD 
		CONSTRAINT [PK_hdAssetUsers] PRIMARY KEY CLUSTERED ([ItemID],[UserID])
end
GO

--adding [Mandatory] and [OrderByNumber] to hdCustomFields
if not exists (select * from syscolumns where id=object_id('hdCustomFields') and name='Mandatory')
begin
	ALTER TABLE hdCustomFields ADD [Mandatory] bit NOT NULL DEFAULT (0)
	ALTER TABLE hdCustomFields ADD [OrderByNumber] int NOT NULL DEFAULT (100)
end
GO

if exists (select * from syscolumns where id=object_id('hdSettings') and name='SendHtmlEmails')
begin
	--dropping default constraints and then columns
	declare @default sysname, @sql nvarchar(max)
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'SendHtmlEmails')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end

	ALTER TABLE hdSettings DROP COLUMN SendHtmlEmails
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='FromName')
begin
	  ALTER TABLE hdSettings ADD FromName varchar(50) NULL
end

--more assets fields
if not exists (select * from syscolumns where id=object_id('hdAssets') and name='SerialNumber')
begin
	ALTER TABLE hdAssets ADD
		SerialNumber nvarchar(200) NULL,
		Location nvarchar(500) NULL,
		Comments nvarchar(1000) NULL
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableKB')
begin
	  ALTER TABLE hdSettings ADD DisableKB bit NOT NULL DEFAULT(0)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AutoLoginSharedSecret')
begin
	  ALTER TABLE hdSettings ADD AutoLoginSharedSecret varchar(20) NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MenuBarTextColor')
begin
	  ALTER TABLE hdSettings ADD MenuBarTextColor char(6) NOT NULL DEFAULT('C7D4E2')
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Disabled')
begin
	  ALTER TABLE hdUsers ADD Disabled bit not null default 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableCloseNotification')
begin
	  ALTER TABLE hdSettings ADD DisableCloseNotification bit NOT NULL DEFAULT 1
end

if not exists (select * from syscolumns where id=object_id('hdCategories') and name='ForTechsOnly')
begin
	ALTER TABLE hdCategories ADD  ForTechsOnly bit NOT NULL DEFAULT (0)
end

--adding commant teamplets table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdCommentTemplates') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdCommentTemplates] (
		[TemplateID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] int NOT NULL DEFAULT(0),
		[Body] nvarchar(max) NOT NULL
	)
END
GO

--converting filenames to nvarchar from varchar
if exists(select name from syscolumns where id=object_id('hdFileAttachments') and name='FileName' and length<255)
	alter table hdFileAttachments alter column FileName nvarchar(255) NOT NULL

--adding "Companies" table and moving data to it
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdCompanies') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdCompanies] (
		CompanyID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] int NOT NULL DEFAULT(0),
		[Name] nvarchar(255) NOT NULL
	)
	
	ALTER TABLE hdUsers ADD CompanyID int NULL
	
	if exists (select * from syscolumns where id=object_id('hdUsers') and name='Company')
	begin
		declare @sql nvarchar(max)
		set @sql = N'INSERT INTO hdCompanies (InstanceID, [Name])
		SELECT DISTINCT InstanceID, Company FROM hdUsers WHERE Company IS NOT NULL AND RTRIM(Company)<>'''''
		exec sp_executesql @sql
	
		set @sql = N'UPDATE hdUsers SET CompanyID = hdCompanies.CompanyID
		from hdUsers
		inner join hdCompanies ON hdUsers.Company=hdCompanies.Name
		where hdUsers.Company IS NOT NULL'
		exec sp_executesql @sql
	end
END
GO

if exists (select * from syscolumns where id=object_id('hdUsers') and name='Company')
begin
	ALTER TABLE hdUsers DROP COLUMN Company
end

--adding "POPServers" table and moving data to it
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdPOPServers') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.hdPOPServers (
		ServerID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] int NOT NULL DEFAULT(0),
		HostName varchar (50) NULL,
		Port int NOT NULL DEFAULT (110),
		POPLogin varchar (50) NULL,
		POPPassword varchar (50) NULL,
		UseSSL bit NOT NULL DEFAULT (0)
	)
	declare @sql nvarchar(max)
	set @sql = N'INSERT INTO hdPOPServers (InstanceID, HostName, Port, POPLogin, POPPassword, UseSSL)
	SELECT InstanceID, MailCheckerPOPServer, MailCheckerPOPPort, MailCheckerPOPLogin, MailCheckerPOPPassword, MailCheckerUseSSL
	FROM hdSettings
	WHERE MailCheckerPOPServer IS NOT NULL AND MailCheckerPOPServer<>'''''
	exec sp_executesql @sql
END
GO
--removing unneeded POP cols from hdsettings
if exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerPOPServer') begin
	--dropping default constraints and then columns
	declare @default sysname, @sql nvarchar(max)
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'MailCheckerPOPPort')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end

	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'MailCheckerUseSSL')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end

	ALTER TABLE hdSettings DROP COLUMN MailCheckerPOPServer, MailCheckerPOPPort, MailCheckerPOPLogin, MailCheckerPOPPassword, MailCheckerUseSSL
end

if not exists (select * from syscolumns where id=object_id('hdPOPServers') and name='NewTicketsCategoryID')
begin
	ALTER TABLE hdPOPServers ADD NewTicketsCategoryID int NULL
end

if not exists (select * from syscolumns where id=object_id('hdComments') and name='IsSystem')
begin
	ALTER TABLE hdComments ADD IsSystem bit NOT NULL DEFAULT(0)
end

if not exists (select * from syscolumns where id=object_id('Instances') and name='CustomDomain')
begin
	ALTER TABLE Instances ADD CustomDomain varchar(100) NULL
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='Title')
begin
	ALTER TABLE hdSettings ADD Title nvarchar(500) NOT NULL DEFAULT ''
end
GO
--moving from instances if any
if exists (select * from syscolumns where id=object_id('Instances') and name='Description')
begin
	declare @sql nvarchar(max)
	set @sql = N'update hdSettings SET hdSettings.Title = Instances.Description
	from Instances inner join hdSettings ON Instances.InstanceID=hdSettings.InstanceID'
	exec sp_executesql @sql

	ALTER TABLE Instances DROP COLUMN Description
end

--adding hdLinkedIssues table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdLinkedIssues') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.hdLinkedIssues (
		Issue1ID int NOT NULL,
		Issue2ID int NOT NULL
	)
	ALTER TABLE hdLinkedIssues WITH NOCHECK ADD 
		CONSTRAINT [PK_hdLinkedIssues] PRIMARY KEY CLUSTERED ([Issue1ID],[Issue2ID])
END
GO

--adding hdIssueSchedule table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdIssueSchedule') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.hdIssueSchedule (
		[IssueID] int NOT NULL PRIMARY KEY,
		FreqType int NOT NULL,
		FirstRecurrenceDate datetime NOT NULL,
		PrevRepeatsCount int NOT NULL DEFAULT(0)
	)
	CREATE TABLE dbo.hdScheduledIssues (
		ScheduledIssueID int NOT NULL PRIMARY KEY,
		OriginalIssueID int NOT NULL
	)
END
GO

--adding "OrderByNumber" to hdCategories
if not exists (select * from syscolumns where id=object_id('hdCategories') and name='OrderByNumber')
begin
	ALTER TABLE hdCategories ADD OrderByNumber int NOT NULL DEFAULT 1
	ALTER TABLE hdSections ADD OrderByNumber int NOT NULL DEFAULT 1
end
GO

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='LastSeen')
begin
	ALTER TABLE hdUsers ADD LastSeen datetime null
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableAssets')
begin
	ALTER TABLE hdSettings ADD DisableAssets bit not null default 0
end

--adding hdMergedIssues table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdMergedIssues') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.hdMergedIssues (
		[IssueID] int NOT NULL,
		[IssueMergedID] int NOT NULL
	)
	ALTER TABLE hdMergedIssues WITH NOCHECK ADD CONSTRAINT [PK_hdMergedIssues] PRIMARY KEY CLUSTERED (IssueID, IssueMergedID)
END
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DaysToCloseAfter')
begin
	ALTER TABLE hdSettings ADD DaysToCloseAfter int NOT NULL DEFAULT(0)
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='IPAddress')
begin
	ALTER TABLE hdUsers ADD IPAddress varchar(50) null
	ALTER TABLE hdUsers ADD HostName varchar(255) null
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Lang')
begin
	ALTER TABLE hdUsers ADD Lang varchar(10)  NOT NULL DEFAULT('')
end

if not exists (select * from syscolumns where id=object_id('hdPOPServers') and name='IsImap')
begin
	  ALTER TABLE hdPOPServers ADD IsImap bit NOT NULL DEFAULT 0
end

--adding indexes to hdIssues
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_CategoryID') begin
	CREATE INDEX IX_hdIssues_CategoryID ON hdIssues (CategoryID)
end
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_UserID') begin
	CREATE INDEX IX_hdIssues_UserID ON hdIssues (UserID)
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='UserAgent')
begin
	ALTER TABLE hdUsers ADD UserAgent varchar(255) null
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='DueDate')
begin
	ALTER TABLE hdIssues ADD DueDate datetime NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='UseFromNameForUserOriginatedMsgs')
begin
	ALTER TABLE hdSettings ADD UseFromNameForUserOriginatedMsgs bit NOT NULL default(0)
end

if not exists (select * from syscolumns where id=object_id('hdFileAttachments') and name='CommentID')
begin
	ALTER TABLE hdFileAttachments ADD CommentID int NOT NULL DEFAULT (0)
end

--TimeSpentInSeconds col
if not exists (select * from syscolumns where id=object_id('hdIssues') and name='TimeSpentInSeconds')
begin
	ALTER TABLE hdIssues ADD TimeSpentInSeconds int NOT NULL DEFAULT (0)
end
GO
if exists (select * from syscolumns where id=object_id('hdIssues') and name='HoursSpent')
begin
	exec sp_executesql N'UPDATE hdIssues SET TimeSpentInSeconds=CAST(HoursSpent * 3600 as int)'
	
	--dropping default constraints and then columns
	declare @default sysname, @sql nvarchar(max)
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdIssues') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdIssues') and name = 'HoursSpent')
	if not @default is null
	begin
		set @sql = N'alter table hdIssues drop constraint ' + @default
		exec sp_executesql @sql
	end
	
	ALTER TABLE hdIssues DROP COLUMN HoursSpent
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='HeaderTextColor')
begin
	ALTER TABLE hdSettings ADD HeaderTextColor char(6) NOT NULL DEFAULT('FFFFFF')
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SameCompanyUserPermission')
begin
	ALTER TABLE hdSettings ADD SameCompanyUserPermission bit not null default(0)
end

--adding indexes to hdIssues
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_StatusID') begin
	CREATE INDEX IX_hdIssues_StatusID ON hdIssues (StatusID)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='HidePoweredBy')
begin
	ALTER TABLE hdSettings ADD HidePoweredBy bit not null default(0)
end

--adding hdMergedIssues table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdOnBehalfIssues') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdOnBehalfIssues] (
		[IssueID] [int] NOT NULL PRIMARY KEY,
		[SubmittedByUserID] int NOT NULL
	)
END
GO

--TAGS!
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdTags') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdTags] (TagID int IDENTITY (1, 1) NOT NULL PRIMARY KEY, [Name] nvarchar(200) NOT NULL, InstanceID int NOT NULL)

	CREATE TABLE [dbo].[hdIssueTags] (TagID int NOT NULL, IssueID int NOT NULL)
	ALTER TABLE hdIssueTags WITH NOCHECK ADD CONSTRAINT [PK_hdIssueTags] PRIMARY KEY CLUSTERED (TagID,IssueID)
END

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableAvatars')
begin
	ALTER TABLE hdSettings ADD DisableAvatars bit not null default(0)
end

--UpdatedForTechView col
if not exists (select * from syscolumns where id=object_id('hdIssues') and name='UpdatedForTechView')
begin
	ALTER TABLE hdIssues ADD UpdatedForTechView bit NOT NULL DEFAULT (0)
end

--not an upgrade but a maintenance procedure: remove deleted tickets from the "hdOnBehalfIssues" tbl
delete from hdOnBehalfIssues where IssueID NOT IN (SELECT IssueID FROM hdIssues)

--Name col in CommentTemplates
if not exists (select * from syscolumns where id=object_id('hdCommentTemplates') and name='Name')
begin
	ALTER TABLE hdCommentTemplates ADD Name varchar(50) NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SendAdminEmailOnNewIssue')
begin
	ALTER TABLE hdSettings ADD SendAdminEmailOnNewIssue bit NOT NULL DEFAULT 1
end

--adding index to hdFileAttachments (IssueID)
if not exists (select name FROM sysindexes WHERE name = 'IX_hdFileAttachments_IssueID') begin
	CREATE INDEX IX_hdFileAttachments_IssueID ON hdFileAttachments (IssueID)
end

--deprecated setting
if exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableCaptcha')
begin
	declare @default sysname, @sql nvarchar(max)
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'DisableCaptcha')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end

	ALTER TABLE hdSettings DROP COLUMN DisableCaptcha
end

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdAssetsIssues') AND type in (N'U'))
begin
	CREATE TABLE dbo.hdAssetsIssues	(IssueID int NOT NULL, AssetID int NOT NULL)
	ALTER TABLE [dbo].hdAssetsIssues WITH NOCHECK ADD CONSTRAINT [PK_hdAssetsIssues] PRIMARY KEY CLUSTERED ([IssueID], [AssetID]) 
end

if exists(select name from syscolumns where id=object_id('hdIssues') and name='Subject' and length<1000)
	ALTER TABLE hdIssues ALTER COLUMN Subject nvarchar(1000)

GO

--passwords to md5 (security)
UPDATE hdUsers SET [Password]=LOWER(CONVERT(VARCHAR(32), HashBytes('MD5', [Password]), 2))
WHERE len([Password])<32 and([Password] is not null)

if not exists (select * from syscolumns where id=object_id('hdCustomFields') and name='ShowInGrid')
begin
	  ALTER TABLE hdCustomFields ADD ShowInGrid bit NOT NULL DEFAULT (1)
end

--signature field for users
if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Signature')
begin
	  ALTER TABLE hdUsers ADD [Signature] nvarchar(500) null
end

--new setting
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AutoAssignFirstReplier')
begin
	ALTER TABLE hdSettings ADD AutoAssignFirstReplier bit NOT NULL DEFAULT 1
end

--NotifyAllTechsOnReply
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='NotifyAllTechsOnReply')
begin
	ALTER TABLE hdSettings ADD NotifyAllTechsOnReply bit NOT NULL DEFAULT 0
end

--increase comment template name size
--commented out to not convert to varchar each time
--alter table hdCommentTemplates alter column [Name] varchar(255) NULL

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'UserAvatars') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[UserAvatars] ([UserID] [int] NOT NULL PRIMARY KEY, [ImageData] [image] NULL)
end

--Recipients
if not exists (select * from syscolumns where id=object_id('hdComments') and name='Recipients')
begin
	ALTER TABLE hdComments ADD Recipients nvarchar(3000) NOT NULL DEFAULT ''
end

--GoogleDrivePath
if not exists (select * from syscolumns where id=object_id('hdFileAttachments') and name='GoogleDriveUrl')
begin
	ALTER TABLE hdFileAttachments ADD GoogleDriveUrl varchar(255) NULL
end

--make FileData column nullable
ALTER TABLE hdFileAttachments ALTER COLUMN FileData [image] NULL

--NewTicketNote
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='NewTicketNote')
begin
	ALTER TABLE hdSettings ADD NewTicketNote varchar(max) NULL
end

GO

--move email templates to separate table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'EmailTemplates') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[EmailTemplates] (
		[InstanceID] int NOT NULL DEFAULT(0),
		EmailTemplate nvarchar (max) NOT NULL DEFAULT '',
		EmailSubjectTemplate nvarchar (500) NOT NULL DEFAULT '',
		NewIssueEmailSubject nvarchar(500) NOT NULL DEFAULT '',
		NewIssueEmailTemplate nvarchar(max) NOT NULL DEFAULT '',
		WelcomeEmailSubj nvarchar (500) NOT NULL DEFAULT '',
		WelcomeEmailBody nvarchar (max) NOT NULL DEFAULT '',
		NewIssueConfirmationSubj nvarchar(500) NOT NULL DEFAULT '',
		NewIssueConfirmationBody nvarchar(max) NOT NULL DEFAULT ''
	)

	declare @sql nvarchar(max)
	set @sql = N'INSERT INTO EmailTemplates (InstanceID, EmailTemplate, EmailSubjectTemplate, NewIssueEmailSubject, NewIssueEmailTemplate, NewIssueConfirmationSubj, NewIssueConfirmationBody)
	SELECT InstanceID, EmailTemplate, EmailSubjectTemplate, NewIssueEmailSubject, NewIssueEmailTemplate, NewIssueEmailSubject, NewIssueEmailTemplate FROM hdSettings'
	exec sp_executesql @sql

	--droping defaults then dropping columns
	declare @default sysname
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D' AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'EmailTemplate')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D' AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'EmailSubjectTemplate')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D' AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'NewIssueEmailSubject')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D' AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'NewIssueEmailTemplate')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end
	ALTER TABLE hdSettings DROP COLUMN EmailTemplate
	ALTER TABLE hdSettings DROP COLUMN EmailSubjectTemplate
	ALTER TABLE hdSettings DROP COLUMN NewIssueEmailSubject
	ALTER TABLE hdSettings DROP COLUMN NewIssueEmailTemplate
end

-- add fulltext search indexes
-- if fulltext components are installed
declare @installed sql_variant
select @installed=SERVERPROPERTY('IsFullTextInstalled')
IF(@installed = 1)
begin
	-- creating default catalog
	IF NOT EXISTS (SELECT * FROM sys.fulltext_catalogs WHERE name='hdFulltextCatalog')
	BEGIN
		CREATE FULLTEXT CATALOG hdFulltextCatalog AS DEFAULT
	end
end

-- user greeting template
if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Greeting')
begin
	ALTER TABLE hdUsers ADD Greeting varchar(500) null
end

--user department column
if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Department')
begin
	  ALTER TABLE hdUsers ADD Department nvarchar(500) NOT NULL DEFAULT ''
end

--user is manager column
if not exists (select * from syscolumns where id=object_id('hdUsers') and name='IsManager')
begin
	  ALTER TABLE hdUsers ADD IsManager bit NOT NULL DEFAULT 0
end

--fixing department column
ALTER TABLE hdUsers ALTER COLUMN Department nvarchar(500) NULL

--adding index to hdComments - UserID
if not exists (select name FROM sysindexes WHERE name = 'IX_hdComments_UserID') begin
	CREATE INDEX IX_hdComments_UserID ON hdComments (UserID)
end

--adding index to hdIssues - AssignedToUserID
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_AssignedToUserID') begin
	CREATE INDEX IX_hdIssues_AssignedToUserID ON hdIssues (AssignedToUserID)
end

--automation
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'AutomationRules') AND type in (N'U'))
begin
	CREATE TABLE dbo.AutomationRules (
		RuleID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		RuleXml nvarchar(max)
	)
	CREATE TABLE dbo.OverdueAutomationLog (
		IssueID int NOT NULL,
		RuleID int NOT NULL
	)
	ALTER TABLE OverdueAutomationLog WITH NOCHECK ADD 
		CONSTRAINT [PK_OverdueAutomationLog] PRIMARY KEY CLUSTERED (IssueID, RuleID)

	CREATE TABLE dbo.NonupdatedAutomationLog (
		IssueID int NOT NULL,
		RuleID int NOT NULL
	)
	ALTER TABLE dbo.NonupdatedAutomationLog WITH NOCHECK ADD 
		CONSTRAINT [PK_NonupdatedAutomationLog] PRIMARY KEY CLUSTERED (IssueID, RuleID)
end
GO

--moving KB-tickets to separate table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'KBArticles') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[KBArticles] (
		[ArticleID] [int] NOT NULL PRIMARY KEY,
		[InstanceID] int NOT NULL DEFAULT(0),
		[CategoryID] [int] NOT NULL ,
		[Subject] [nvarchar] (1000) NOT NULL ,
		[Body] nvarchar(max) NOT NULL,
		[ForTechsOnly] [bit] NOT NULL DEFAULT (0)
	)

	CREATE TABLE [dbo].[KBArticleTags] (
		TagID int NOT NULL,
		ArticleID int NOT NULL
	)
	ALTER TABLE KBArticleTags WITH NOCHECK ADD CONSTRAINT [PK_KBArticleTags] PRIMARY KEY CLUSTERED (TagID,ArticleID)

	--moving kb-tickets to articles
	declare @sql nvarchar(max)
	set @sql = N'INSERT INTO KBArticles (ArticleID, InstanceID, CategoryID, [Subject], Body, ForTechsOnly) SELECT IssueID, InstanceID, CategoryID, [Subject], Body, KBForTechsOnly FROM hdIssues WHERE PublishToKb=1'
	exec sp_executesql @sql

	--now moving comments to KB article body
	UPDATE KBArticles
	SET Body = Body + char(10) + ISNULL('[hr]' + (SELECT body + char(10) + char(10) AS [text()] From hdComments where IssueID = KBArticles.ArticleID AND IsSystem=0 AND Body<>'' For XML PATH ('')), '')

	--tags
	set @sql = N'INSERT INTO KBArticleTags (ArticleID, TagID) SELECT IssueID, TagID FROM hdIssueTags WHERE IssueID IN (SELECT IssueID FROM hdIssues WHERE PublishToKb=1)'
	exec sp_executesql @sql
end
GO

IF EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'FK_hdFileAttachments_hdIssues') AND parent_object_id = OBJECT_ID(N'hdFileAttachments'))
	ALTER TABLE hdFileAttachments DROP CONSTRAINT [FK_hdFileAttachments_hdIssues]

--custom field categories
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'CustomFieldCategories') AND type in (N'U'))
begin
	CREATE TABLE dbo.CustomFieldCategories (
		FieldID int NOT NULL,
		CategoryID int NOT NULL
	)
	ALTER TABLE CustomFieldCategories WITH NOCHECK ADD CONSTRAINT [PK_CustomFieldCategories] PRIMARY KEY CLUSTERED (FieldID,CategoryID)
end

if not exists(select name from syscolumns where id=object_id('hdSettings') and name='CustomCSS')
	alter table hdSettings add CustomCSS varchar(max) NULL

if not exists(select name from syscolumns where id=object_id('hdCategories') and name='FromAddress')
	alter table hdCategories add FromAddress varchar(255) NULL

--drop users foreign key contraints from hdFileAttachments
IF EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'FK_hdFileAttachments_hdUsers') AND parent_object_id = OBJECT_ID(N'hdFileAttachments'))
	ALTER TABLE hdFileAttachments DROP CONSTRAINT [FK_hdFileAttachments_hdUsers]

GO

if not exists(select name from syscolumns where id=object_id('hdUsers') and name='TechTutorialShown')
begin
	alter table hdUsers add TechTutorialShown bit not null default(0)
	
	declare @sql nvarchar(max)
	set @sql = 'update hdUsers SET TechTutorialShown=1'
	exec sp_executesql @sql
end

if not exists(select name from syscolumns where id=object_id('hdSettings') and name='DisableAutoTimer')
	alter table hdSettings add DisableAutoTimer bit not null default 0

if not exists(select name from syscolumns where id=object_id('hdComments') and name='EmailHeaders')
	alter table hdComments add EmailHeaders varchar(1000) NULL

if not exists(select name from syscolumns where id=object_id('hdUsers') and name='PushToken')
	alter table hdUsers add PushToken varchar(64) NULL

if not exists(select name from syscolumns where id=object_id('hdSettings') and name='DontAddCCToSubscribers')
	alter table hdSettings add DontAddCCToSubscribers bit not null default 0

if not exists(select name from syscolumns where id=object_id('hdCompanies') and name='Notes')
	alter table hdCompanies add Notes nvarchar(2000) NULL

if not exists (select * from syscolumns where id=object_id('KBArticles') and name='OrderByNumber')
	ALTER TABLE KBArticles ADD OrderByNumber int NOT NULL DEFAULT 1


--creating cat index in KB
if not exists (select name FROM sysindexes WHERE name = 'IX_KBArticles_CategoryID')
	CREATE INDEX IX_KBArticles_CategoryID ON KBArticles (CategoryID)

--working hours
if not exists(select name from syscolumns where id=object_id('hdSettings') and name='WorkHourFrom')
	alter table hdSettings add WorkHourFrom int null, WorkHourTo int null

--closed template
if not exists(select name from syscolumns where id=object_id('EmailTemplates') and name='ClosedTicketEmailTemplate')
	alter table EmailTemplates add ClosedTicketEmailTemplate varchar(max) NOT NULL DEFAULT ''

if not exists(select name from syscolumns where id=object_id('EmailTemplates') and name='ClosedTicketEmailSubject')
	alter table EmailTemplates add ClosedTicketEmailSubject varchar(500) NOT NULL DEFAULT ''

--custom field categories
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'ReportSchedule') AND type in (N'U'))
begin
	CREATE TABLE dbo.ReportSchedule (
		InstanceID int NOT NULL DEFAULT(0) PRIMARY KEY,
		ReportModelXml varchar(max) not null,
		LastSent datetime null
	)
end

if not exists(select name from syscolumns where id=object_id('hdIssueSchedule') and name='ReopenOriginal')
	alter table hdIssueSchedule add ReopenOriginal bit not null default 0

if not exists(select name from syscolumns where id=object_id('hdSettings') and name='EmailOtherTechsOnTakeover')
	alter table hdSettings add EmailOtherTechsOnTakeover bit not null default 0

if not exists(select name from syscolumns where id=object_id('hdSettings') and name='EnableSaml')
begin
	alter table hdSettings add
		EnableSaml bit not null default 0,
		SamlEndpoint varchar(200),
		SamlCertificate varchar(2000)
end
go

if not exists(select name from syscolumns where id=object_id('hdUsers') and name='AdminTutorialShown')
begin
	alter table hdUsers add AdminTutorialShown bit not null default(0)

	declare @sql nvarchar(max)
	set @sql = 'update hdUsers SET AdminTutorialShown=1'
	exec sp_executesql @sql
end

--rename the column properly (for really OLD versions)
if exists(select name from syscolumns where id=object_id('hdCompanies') and name='CompanyName')
begin
	exec sp_rename 'hdCompanies.CompanyName', 'Name', 'COLUMN'
end
go

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableTicketConfirmationNotification')
begin
	  ALTER TABLE hdSettings ADD DisableTicketConfirmationNotification bit NOT NULL DEFAULT 0
end
go

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='TimezoneID')
begin
	ALTER TABLE hdSettings ADD TimezoneID varchar(50) NOT NULL default('Eastern Standard Time')

	-- first let's convert server time offset to the UTC offset, so that we can work with it
	-- now we can convert it to a timezone
	declare @sql nvarchar(max) 

	set @sql = 'UPDATE hdSettings SET TimezoneID = CASE ServerTimeOffset + datediff(hour,GETUTCDATE(), getdate())
	WHEN -12 THEN ''Dateline Standard Time''
	WHEN -11 THEN ''UTC-11''
	WHEN -10 THEN ''Hawaiian Standard Time''
	WHEN -9 THEN ''Alaskan Standard Time''
	WHEN -8 THEN ''Pacific Standard Time''
	WHEN -7 THEN ''Mountain Standard Time''
	WHEN -6 THEN ''Central Standard Time''
	WHEN -5 THEN ''Eastern Standard Time''
	WHEN -4 THEN ''Atlantic Standard Time''
	WHEN -3 THEN ''E. South America Standard Time''
	WHEN -2 THEN ''UTC-02''
	WHEN -1 THEN ''Cape Verde Standard Time''
	WHEN 0 THEN ''GMT Standard Time''
	WHEN 1 THEN ''W. Europe Standard Time''
	WHEN 2 THEN ''Middle East Standard Time''
	WHEN 3 THEN ''Arabic Standard Time''
	WHEN 4 THEN ''Russian Standard Time''
	WHEN 5 THEN ''West Asia Standard Time''
	WHEN 6 THEN ''Central Asia Standard Time''
	WHEN 7 THEN ''SE Asia Standard Time''
	WHEN 8 THEN ''China Standard Time''
	WHEN 9 THEN ''Tokyo Standard Time''
	WHEN 10 THEN ''AUS Eastern Standard Time''
	WHEN 11 THEN ''Vladivostok Standard Time''
	WHEN 12 THEN ''UTC+12''
	ELSE ''Eastern Standard Time'' END'
	
	exec sp_executesql @sql

	--dropping default constraints and then columns
	declare @default sysname
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'ServerTimeOffset')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
		set @sql = 'alter table hdSettings drop column ServerTimeOffset'
		exec sp_executesql @sql 
	end
end
go

if not exists (select * from syscolumns where id=object_id('hdStatus') and name='ForTechsOnly')
begin
	  ALTER TABLE hdStatus ADD ForTechsOnly bit NOT NULL DEFAULT 1
end

if not exists (select * from syscolumns where id=object_id('hdCustomFields') and name='UsageType')
begin
	  ALTER TABLE hdCustomFields ADD UsageType int NOT NULL DEFAULT 1
end

--assets custom field values
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdCustomFieldValuesForAssets') AND type in (N'U'))
begin
	declare @sql nvarchar(max)
	set @sql = N'CREATE TABLE dbo.hdCustomFieldValuesForAssets (
		[ItemID] [int] NOT NULL,
		[FieldID] [int] NOT NULL,
		[Value] sql_variant NOT NULL)'
	exec sp_executesql @sql

	set @sql = N'ALTER TABLE [dbo].[hdCustomFieldValuesForAssets] WITH NOCHECK ADD 
		CONSTRAINT [PK_hdCustomFieldValuesForAssets] PRIMARY KEY CLUSTERED ([ItemID],[FieldID])'
	exec sp_executesql @sql	
end

if not exists (select * from syscolumns where id=object_id('AutomationRules') and name='OrderByNumber')
begin
	ALTER TABLE AutomationRules ADD OrderByNumber int NOT NULL DEFAULT 1
end
GO

--Integrations
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'Integrations') AND type in (N'U'))
begin
	CREATE TABLE dbo.Integrations (
		[InstanceID] int NOT NULL PRIMARY KEY,
		BitbucketEnabled bit NOT NULL DEFAULT 0,
		BitbucketUsername varchar(50) null,
		BitbucketPassword varchar(50) null,
		HipChatEnabled bit NOT NULL DEFAULT 0,
		HipChatAuthToken varchar(50) null,
		HipChatRoomId int null,
		JiraEnabled bit NOT NULL DEFAULT 0,
		JiraUsername varchar(50) null,
		JiraPassword varchar(500) null,
		JiraUrl varchar(100) null
	)

	CREATE TABLE dbo.IntegrationTicketData (
		IssueID int NOT NULL PRIMARY KEY,
		BitbucketIssueUrl varchar(300) null,
		JiraIssueUrl varchar(300) null
	)
end

--Dropbox
if not exists (select * from syscolumns where id=object_id('Integrations') and name='DropboxEnabled')
begin
	ALTER TABLE Integrations ADD DropboxEnabled bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='DropboxAppID')
begin
	ALTER TABLE Integrations ADD DropboxAppID varchar(25) NULL
	
	--dropbox was not working anyway, so lets force people to go to Admin
	declare @sql nvarchar(max)
	set @sql = 'UPDATE Integrations SET DropboxEnabled=0'
	exec sp_executesql @sql
end

--JiraIssueUrl - some client reported bugs
if not exists (select * from syscolumns where id=object_id('IntegrationTicketData') and name='JiraIssueUrl')
begin
	ALTER TABLE IntegrationTicketData ADD JiraIssueUrl varchar(300) null
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='ActiveTabTextColor')
begin
	  ALTER TABLE hdSettings ADD ActiveTabTextColor char(6) NOT NULL DEFAULT('FFFFFF')
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='SlackEnabled')
begin
	ALTER TABLE Integrations ADD SlackEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE Integrations ADD SlackWebhookURL varchar(100) NULL
end

if not exists (select * from syscolumns where id=object_id('Instances') and name='WidgetInstalled')
begin
	ALTER TABLE Instances ADD WidgetInstalled bit NOT NULL DEFAULT 0
end
GO

if not exists (select * from syscolumns where id=object_id('hdFileAttachments') and name='FileHash')
begin
	ALTER TABLE hdFileAttachments ADD FileHash binary(16) NULL
	ALTER TABLE hdFileAttachments ADD FileSize int NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdCustomFieldOptions') and name='IsDefault')
begin
	ALTER TABLE hdCustomFieldOptions ADD IsDefault bit not null default 0
end

--converting hdUsers field to nvarchar and Status from varchar to Nvarchar
if exists (select id from syscolumns where id=object_id('hdUsers') and name='Location' and type_name(xtype)<>'nvarchar')
	alter table hdUsers alter column Location nvarchar(1000) NULL
if exists (select id from syscolumns where id=object_id('hdUsers') and name='Notes' and type_name(xtype)<>'nvarchar')
begin
	--droping defaults then dropping columns
	declare @default sysname, @sql nvarchar(max)
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdUsers') AND type = 'D' AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdUsers') and name = 'Notes')
	if not @default is null
	begin
		set @sql = N'alter table hdUsers drop constraint ' + @default
		exec sp_executesql @sql
	end

	alter table hdUsers alter column Notes nvarchar(4000) NULL
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='GitHubEnabled')
begin
	ALTER TABLE Integrations ADD GitHubEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE Integrations ADD GitHubUsername varchar(50) null
	ALTER TABLE Integrations ADD GitHubPassword varchar(50) null
	ALTER TABLE IntegrationTicketData ADD GitHubIssueUrl varchar(300) null
end

if exists (select id from syscolumns where id=object_id('hdSettings') and name='NewTicketNote' and type_name(xtype)<>'nvarchar')
begin
	alter table hdSettings alter column NewTicketNote nvarchar(max) NULL
end
GO

--Category submit permissions
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdCategorySubmitPermissions') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[hdCategorySubmitPermissions] (
		[UserID] [int] NOT NULL ,
		[CategoryID] [int] NOT NULL 
	)
	ALTER TABLE [dbo].[hdCategorySubmitPermissions] WITH NOCHECK ADD 
		CONSTRAINT [PK_hdCategorySubmitPermissions] PRIMARY KEY CLUSTERED ([UserID],[CategoryID])

	ALTER TABLE hdCategories ADD ForSpecificUsers bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='RemoteLoginURL')
begin
	  ALTER TABLE hdSettings ADD RemoteLoginURL varchar(500) NULL
end
GO

--Category submit permissions BY COMPANY
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdCategorySubmitPermissionsByCompany') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[hdCategorySubmitPermissionsByCompany] (
		[CompanyID] [int] NOT NULL ,
		[CategoryID] [int] NOT NULL 
	)
	ALTER TABLE [dbo].[hdCategorySubmitPermissionsByCompany] WITH NOCHECK ADD 
		CONSTRAINT [PK_hdCategorySubmitPermissionsByCompany] PRIMARY KEY CLUSTERED([CompanyID],[CategoryID])
end

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'DeletionLog') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[DeletionLog](
		[InstanceID] INT NOT NULL, 
		[UserID] INT NOT NULL, 
		[Description] NVARCHAR(500) NOT NULL,
		[Date] [datetime] NOT NULL DEFAULT (getdate())
	)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='NoAttachmentsInNotifications')
begin
	  ALTER TABLE hdSettings ADD NoAttachmentsInNotifications bit NOT NULL DEFAULT 0
end
GO

if not exists (select * from syscolumns where id=object_id('Integrations') and name='UserLookupEnabled')
begin
	ALTER TABLE Integrations ADD UserLookupEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE Integrations ADD UserLookupURL varchar(500) NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableAutologinLinks')
begin
	  ALTER TABLE hdSettings ADD DisableAutologinLinks bit NOT NULL DEFAULT 0
end
GO

if not exists (select * from syscolumns where id=object_id('DeletionLog') and name='LogID')
begin
	  ALTER TABLE DeletionLog ADD LogID int IDENTITY (1, 1) NOT NULL PRIMARY KEY
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='ExtractOriginalSenderFromFwd')
begin
	  ALTER TABLE hdSettings ADD ExtractOriginalSenderFromFwd bit NOT NULL DEFAULT 1
end
GO

if not exists (select * from syscolumns where id=object_id('Integrations') and name='AsanaEnabled')
begin
	ALTER TABLE Integrations ADD AsanaEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE Integrations ADD AsanaAccessToken varchar(300) null
	ALTER TABLE IntegrationTicketData ADD AsanaTaskUrl varchar(300) null
end

if not exists (select * from syscolumns where id=object_id('hdCategories') and name='KBOnly')
begin
	  ALTER TABLE hdCategories ADD KBOnly bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdCategories') and name='FromAddressInReplyTo')
begin
	ALTER TABLE hdCategories ADD FromAddressInReplyTo bit NOT NULL DEFAULT 1
	
	declare @sql nvarchar(max)
	set @sql = N'UPDATE hdCategories SET FromAddressInReplyTo = 0' --set to FALSE for existing categories
	exec sp_executesql @sql
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='FaviconImage')
begin
	  ALTER TABLE hdSettings ADD FaviconImage [image] NULL
end


if not exists (select * from syscolumns where id=object_id('KBArticles') and name='DateCreated')
begin
	  ALTER TABLE KBArticles ADD DateCreated datetime NULL DEFAULT (getdate())
	  ALTER TABLE KBArticles ADD LastUpdated datetime NULL DEFAULT (getdate())
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='EnableGoogleLogin')
begin
	  ALTER TABLE hdSettings ADD EnableGoogleLogin bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdCategories') and name='Notes')
begin
	ALTER TABLE hdCategories ADD Notes nvarchar(max) NULL
end

GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'Departments') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[Departments] (
	DepartmentID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[Name] nvarchar(255) NOT NULL
	)

	ALTER TABLE hdUsers ADD DepartmentID int null

	declare @sql nvarchar(max)
	set @sql = N'INSERT INTO Departments (InstanceID, Name)
		SELECT DISTINCT InstanceID, LTRIM(RTRIM(Department)) COLLATE SQL_Latin1_General_CP1_CI_AI FROM hdUsers WHERE Department is not null AND Department <> '''''
	exec sp_executesql @sql

	set @sql = N'UPDATE hdUsers
		SET DepartmentID = (SELECT DepartmentID FROM Departments WHERE Departments.Name = hdUsers.Department AND Departments.InstanceID = hdUsers.InstanceID)
		WHERE Department is not null AND Department <> '''''
	exec sp_executesql @sql

	--ALTER TABLE hdUsers DROP COLUMN Department
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='TwoFactorAuthEnabled')
begin
	  ALTER TABLE hdUsers ADD TwoFactorAuthEnabled bit NOT NULL DEFAULT (0)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableUsernameEmailEdit')
begin
	  ALTER TABLE hdSettings ADD DisableUsernameEmailEdit bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdPOPServers') and name='Disabled')
begin
	  ALTER TABLE hdPOPServers ADD Disabled bit NOT NULL DEFAULT 0
	  ALTER TABLE hdPOPServers ADD ImapMarkSeenNotDelete bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdCustomFieldOptions') and name='OrderByNumber')
begin
	ALTER TABLE hdCustomFieldOptions ADD OrderByNumber int NOT NULL DEFAULT 1
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='TrelloEnabled')
begin
	ALTER TABLE Integrations ADD TrelloEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE IntegrationTicketData ADD TrelloCardUrl varchar(300) null
end

if not exists (select * from syscolumns where id=object_id('hdFileAttachments') and name='HiddenFromKB')
begin
	ALTER TABLE hdFileAttachments ADD HiddenFromKB bit NOT NULL DEFAULT 0
	ALTER TABLE hdFileAttachments ADD HiddenFromTickets bit NOT NULL DEFAULT 0
end


IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'StatusCategories') AND type in (N'U'))
begin
	CREATE TABLE dbo.StatusCategories (
		StatusID int NOT NULL,
		[CategoryID] int NOT NULL
	)
	ALTER TABLE StatusCategories WITH NOCHECK ADD CONSTRAINT [PK_StatusCategories] PRIMARY KEY CLUSTERED (StatusID,CategoryID)
end
go

--fix for incorrect deletion procedure
delete from hdAssetsIssues where IssueID not in (select IssueID from hdIssues)
delete from hdAssetUsers where UserID not in (select UserID from hdUsers)
go

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='KBTitle')
begin
	  ALTER TABLE hdSettings ADD KBTitle nvarchar(max) NULL, KBCss varchar(max) NULL, KBHeadCode varchar(max) NULL
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='VSOEnabled')
begin
	ALTER TABLE Integrations ADD VSOEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE Integrations ADD VSOAccountName varchar(50) null
	ALTER TABLE Integrations ADD VSOUsername varchar(50) null
	ALTER TABLE Integrations ADD VSOToken varchar(100) null
	ALTER TABLE IntegrationTicketData ADD VSOWorkItemUrl varchar(300) null
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='HarvestEnabled')
begin
	ALTER TABLE Integrations ADD HarvestEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE Integrations ADD HarvestUsername varchar(50) null
	ALTER TABLE Integrations ADD HarvestPassword varchar(50) null
	ALTER TABLE Integrations ADD HarvestAccount varchar(100) null
	ALTER TABLE IntegrationTicketData ADD HarvestProjectUrl varchar(300) null
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SendTechEmailOnKBUpdate')
begin
	ALTER TABLE hdSettings ADD SendTechEmailOnKBUpdate bit not null default 0
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='Origin')
begin
	ALTER TABLE hdIssues ADD Origin smallint NOT NULL default(0)

	--update origin for existing tickets (5 months old)
	declare @sql nvarchar(max)
	set @sql = N'update hdIssues SET Origin = case when hdComments.EmailHeaders is not null then 1 when hdComments.Body like ''%[b]widget[/b]%'' then 2 else 0 end 
		from hdIssues
		inner join hdComments ON hdComments.CommentID = (select top 1 CommentID from hdComments where IssueID = hdIssues.IssueID order by commentId)
		where hdIssues.IssueDate>getdate()-150'
	exec sp_executesql @sql
end
go

alter table Integrations alter column AsanaAccessToken nvarchar(300) NULL

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='HideLoginButtonWhenSAML')
begin
	ALTER TABLE hdSettings ADD HideLoginButtonWhenSAML bit not null default 0
end

if not exists (select * from syscolumns where id=object_id('hdCustomFieldOptions') and name='IsDisabled')
begin
	ALTER TABLE hdCustomFieldOptions ADD IsDisabled bit not null default 0
end

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'ZapierSubscriptions') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.ZapierSubscriptions
	(
		[SubscriptionID] INT IDENTITY (1, 1) NOT NULL PRIMARY KEY, 
		[InstanceID] INT NOT NULL, 
		[EventID] INT NOT NULL, 
		[TargetURL] VARCHAR(500) NOT NULL
	)

	ALTER TABLE Integrations ADD ZapierEnabled bit not null default 0
END
GO

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='TimezoneID')
begin
	ALTER TABLE hdUsers ADD TimezoneID varchar(50) NULL
end

alter table hdUsers alter column [Signature] nvarchar(2000) NULL

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='CustomJS')
begin
	ALTER TABLE hdSettings ADD CustomJS varchar(max) NULL
end

if not exists (select * from syscolumns where id=object_id('hdStatus') and name='StopTimeSpent')
begin
	  ALTER TABLE hdStatus ADD StopTimeSpent bit NOT NULL DEFAULT 0
end

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'DuplicateFiles') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.DuplicateFiles (
		FileID int NOT NULL,
		CommentID int NOT NULL
	)
	ALTER TABLE DuplicateFiles WITH NOCHECK ADD
		CONSTRAINT [PK_DuplicateFiles] PRIMARY KEY CLUSTERED (FileID, CommentID)
END
GO

--increase Jira URL size
alter table Integrations alter column JiraUrl varchar(100) NULL

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='LastPasswordChange')
begin
	ALTER TABLE hdUsers ADD LastPasswordChange datetime NULL
end
GO

--change to nvarchar to support non-english chars
alter table hdCommentTemplates alter column [Name] nvarchar(255) NULL

--multiassign
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='EnableMultiAsssign')
begin
	ALTER TABLE hdSettings ADD EnableMultiAsssign bit not null default 0
end
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'SecondaryAssignees') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.SecondaryAssignees (
		IssueID int NOT NULL,
		UserID int NOT NULL
	)
	ALTER TABLE [dbo].[SecondaryAssignees] WITH NOCHECK ADD 
		CONSTRAINT [PK_SecondaryAssignees] PRIMARY KEY CLUSTERED ([IssueID],[UserID])
END
GO

if not exists (select * from syscolumns where id=object_id('Integrations') and name='UserLookupBasicAuthEnabled')
begin
	ALTER TABLE Integrations ADD UserLookupBasicAuthEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE Integrations ADD UserLookupUsername varchar(50) NULL
	ALTER TABLE Integrations ADD UserLookupPassword varchar(50) NULL
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='GoogleDriveEnabled')
begin
	ALTER TABLE Integrations ADD GoogleDriveEnabled bit not null default 0
end

if not exists (select * from syscolumns where id=object_id('hdFileAttachments') and name='AssetID')
begin
	ALTER TABLE hdFileAttachments ADD AssetID int null
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='GithubEnterpriseUrl')
begin
	ALTER TABLE Integrations ADD GithubEnterpriseUrl varchar(100) null
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='LogoURL')
begin
	ALTER TABLE hdSettings ADD LogoURL varchar(100) NULL
end

if not exists (select * from syscolumns where id=object_id('hdPOPServers') and name='Protocol')
begin
	ALTER TABLE hdPOPServers ADD Protocol int not null default(0)

	declare @sql nvarchar(max)
	set @sql = N'UPDATE hdPOPServers SET Protocol = IsImap'
	exec sp_executesql @sql
end

GO

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='SendNewTicketTechEmail')
begin
	ALTER TABLE hdUsers ADD SendNewTicketTechEmail bit not null DEFAULT (1)
end

--creating KB subscriptions table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'KBSubscribers') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.KBSubscribers (
		CategoryID int NOT NULL,
		UserID int NOT NULL
	)
	ALTER TABLE KBSubscribers WITH NOCHECK ADD
		CONSTRAINT [PK_KBSubscribers] PRIMARY KEY CLUSTERED (CategoryID, UserID)
END
GO

--adding "OptionID" to custom field value, for dropdown fields
if not exists (select * from syscolumns where id=object_id('hdCustomFieldValues') and name='OptionID')
begin
	ALTER TABLE hdCustomFieldValues ADD OptionID int NULL
	ALTER TABLE hdCustomFieldValuesForAssets ADD OptionID int NULL

	declare @sql nvarchar(max)
	set @sql = N'update hdCustomFieldValuesForAssets
				set OptionID = CAST(value as int)
				where FieldID in (select FieldID from hdCustomFields where Type=3) and SQL_VARIANT_PROPERTY(value, ''BaseType'')=''int'' and optionid is null'
	exec sp_executesql @sql

	set @sql = N'update hdCustomFieldValues
				set OptionID = CAST(value as int)
				where FieldID in (select FieldID from hdCustomFields where Type=3) and SQL_VARIANT_PROPERTY(value, ''BaseType'')=''int'' and optionid is null'
	exec sp_executesql @sql
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DaysToAllowReopen')
begin
	ALTER TABLE hdSettings ADD DaysToAllowReopen int NOT NULL DEFAULT 30

	declare @sql nvarchar(max)

	set @sql = N'UPDATE hdSettings SET DaysToAllowReopen = 0 WHERE DontReopenOnReply = 1'
	exec sp_executesql @sql
	set @sql = N'UPDATE hdSettings SET DaysToAllowReopen = -1 WHERE DontReopenOnReply = 0'
	exec sp_executesql @sql

	--dropping default constraints and then columns
	declare @default sysname
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'DontReopenOnReply')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
		set @sql = 'alter table hdSettings drop column DontReopenOnReply'
		exec sp_executesql @sql 
	end
end
GO

if not exists (select * from syscolumns where id=object_id('hdFileAttachments') and name='InstanceID')
begin
	ALTER TABLE hdFileAttachments ADD InstanceID int NOT NULL DEFAULT 0
end

--converting RuleXml in AutomationRules to Nvarchar
if exists (select id from syscolumns where id=object_id('AutomationRules') and name='RuleXml' and type_name(xtype)<>'nvarchar')
	alter table AutomationRules alter column RuleXml nvarchar(max) NULL

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'CompanyDomains') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[CompanyDomains] (
		CompanyID int NOT NULL,
		Domain nvarchar(100) NOT NULL
	)
	ALTER TABLE CompanyDomains WITH NOCHECK ADD
		CONSTRAINT [PK_CompanyDomains] PRIMARY KEY CLUSTERED (CompanyID, Domain)

	if exists (select * from syscolumns where id=object_id('hdCompanies') and name='EmailDomain')
	begin
		declare @sql nvarchar(max)
		set @sql = N'INSERT INTO CompanyDomains (CompanyID, Domain)
		SELECT CompanyID, LTRIM(RTRIM(m.n.value(''.[1]'',''nvarchar(100)''))) AS Domain
		FROM (
			SELECT CompanyID,CAST(''<XMLRoot><RowData>'' + REPLACE(EmailDomain,'';'',''</RowData><RowData>'') + ''</RowData></XMLRoot>'' AS XML) AS x
			FROM   hdCompanies WHERE EmailDomain is not null and EmailDomain<>'''' 	
		)t CROSS APPLY x.nodes(''/XMLRoot/RowData'')m(n)'
		exec sp_executesql @sql

	end
END
GO

if exists (select * from syscolumns where id=object_id('hdCompanies') and name='EmailDomain')
begin
	ALTER TABLE hdCompanies DROP COLUMN EmailDomain
end

if exists (select id from syscolumns where id=object_id('hdFileAttachments') and name='FileData' and type_name(xtype)<>'varbinary')
	alter table hdFileAttachments alter column FileData varbinary(max) NULL

--trash
if not exists (select * from hdStatus where StatusID=-1)
begin
	SET IDENTITY_INSERT hdStatus ON
	INSERT INTO hdStatus (StatusID, Name) VALUES (-1, 'Deleted')
	SET IDENTITY_INSERT hdStatus OFF
end

if not exists (select * from syscolumns where id=object_id('hdCategories') and name='FromName')
begin
	ALTER TABLE hdCategories ADD FromName varchar(255) NULL
end

-- add fulltext KB indexes
declare @installedfts sql_variant
select @installedfts=SERVERPROPERTY('IsFullTextInstalled')
IF(@installedfts = 1)
begin
	IF NOT EXISTS (select * from sys.fulltext_indexes where object_id=object_id('KBArticles'))
	BEGIN
		declare @kbConstraint sysname
		declare @kbIndexCmd varchar(1000)

		SELECT @kbConstraint=CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME='KBArticles' AND CONSTRAINT_TYPE='PRIMARY KEY'

		SET @kbIndexCmd = 'CREATE FULLTEXT INDEX ON KBArticles (Subject, [Body])
								KEY INDEX ' + @kbConstraint
						
		EXEC(@kbIndexCmd)
	end
end

if not exists (select * from syscolumns where id=object_id('EmailTemplates') and name='UnregUserBounceSubj')
begin
	ALTER TABLE EmailTemplates ADD UnregUserBounceSubj nvarchar(500) NOT NULL DEFAULT ''
	ALTER TABLE EmailTemplates ADD UnregUserBounceBody nvarchar(max) NOT NULL DEFAULT ''
end

--KB suggestions cache
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'KBSuggestionsCache') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[KBSuggestionsCache] (
		IssueID int NOT NULL,
		ArticleID int NOT NULL
	)
	ALTER TABLE KBSuggestionsCache WITH NOCHECK ADD CONSTRAINT [PK_KBSuggestions] PRIMARY KEY CLUSTERED (
		IssueID,
		ArticleID
	)
END
GO

--add sme missing indexes
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_IssueDate')
	CREATE INDEX IX_hdIssues_IssueDate ON hdIssues (IssueDate)
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_AssignedToUserID')
	CREATE INDEX IX_hdIssues_AssignedToUserID ON hdIssues (AssignedToUserID)
if not exists (select name FROM sysindexes WHERE name = 'IX_hdCategoryPermissions_UserID')
	CREATE INDEX IX_hdCategoryPermissions_UserID ON hdCategoryPermissions (UserID)
if not exists (select name FROM sysindexes WHERE name = 'IX_hdCustomFieldOptions_FieldID')
	CREATE INDEX IX_hdCustomFieldOptions_FieldID ON hdCustomFieldOptions (FieldID)

if not exists (select * from syscolumns where id=object_id('KBArticles') and name='AIMeta')
begin
	ALTER TABLE KBArticles ADD AIMeta nvarchar(max) NULL
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='HighriseEnabled')
begin
	ALTER TABLE Integrations ADD HighriseEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE Integrations ADD HighriseUrl varchar(150) NULL
	ALTER TABLE Integrations ADD HighriseToken varchar(50) NULL
end

if not exists (select * from syscolumns where id=object_id('hdIssueSchedule') and name='LastRecurrenceDate')
begin
	ALTER TABLE hdIssueSchedule ADD LastRecurrenceDate datetime null
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SamlEntityId')
begin
	ALTER TABLE hdSettings ADD SamlEntityId varchar(200)
end
go

if not exists (select * from syscolumns where id=object_id('hdAssets') and name='CompanyID')
begin
	ALTER TABLE hdAssets ADD CompanyID int null
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='KBBotEnabled')
begin
	  ALTER TABLE hdSettings ADD KBBotEnabled bit NOT NULL default(1)
end
go

if not exists (select * from syscolumns where id=object_id('hdStatus') and name='AccessType')
begin
	ALTER TABLE hdStatus ADD AccessType tinyint NOT NULL default(0)

	declare @sql nvarchar(max)
	set @sql = N'UPDATE hdStatus SET AccessType = ForTechsOnly WHERE InstanceID IS NOT NULL'
	exec sp_executesql @sql
end
go

if not exists (select * from syscolumns where id=object_id('hdCustomFields') and name='AccessType')
begin
	ALTER TABLE hdCustomFields ADD AccessType tinyint NOT NULL default(0)

	declare @sql nvarchar(max)
	set @sql = N'UPDATE hdCustomFields SET AccessType = ForTechsOnly WHERE InstanceID IS NOT NULL'
	exec sp_executesql @sql
end

--nested automation rules
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'AutomationQueue') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.AutomationQueue (
		IssueID int NOT NULL,
		RuleID int NOT NULL
	)
	ALTER TABLE AutomationQueue WITH NOCHECK ADD 
		CONSTRAINT [PK_AutomationQueue] PRIMARY KEY CLUSTERED (IssueID, RuleID)
END
GO

if not exists(select name from syscolumns where id=object_id('hdSettings') and name='KBHomeArticleCount')
	alter table hdSettings add KBHomeArticleCount tinyint NOT NULL default 5

GO

--creating KB subscriptions to individual articles
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'KBArticleSubscribers') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.KBArticleSubscribers (
		ArticleID int NOT NULL,
		UserID int NOT NULL
	)
	ALTER TABLE KBArticleSubscribers WITH NOCHECK ADD
		CONSTRAINT [PK_KBArticleSubscribers] PRIMARY KEY CLUSTERED (ArticleID, UserID)
END
GO

if not exists(select name from syscolumns where id=object_id('hdAssets') and name='Disabled')
	alter table hdAssets add [Disabled] bit NOT NULL default 0

if not exists(select name from syscolumns where id=object_id('KBArticles') and name='CreatedByUserID')
begin
	alter table KBArticles add CreatedByUserID int null
	alter table KBArticles add LastUpdatedByUserID int null
end

--asset log
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'AssetLog') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[AssetLog] (
		[LogID]        INT           IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[ItemID]       INT           NOT NULL,
		[OriginatorID] INT           NOT NULL,
		[Description]  VARCHAR (500) NOT NULL,
		[EntryDate]    DATETIME      DEFAULT (getdate()) NOT NULL
	)
END
GO

if not exists (select * from syscolumns where id=object_id('Integrations') and name='VSOSelfHosted')
begin
	ALTER TABLE Integrations ADD VSOSelfHosted bit NOT NULL DEFAULT 0
end
go

--ideas
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'Ideas') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.Ideas (
		IdeaID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] int NOT NULL DEFAULT(0),
		[Subject] [nvarchar] (1000) NOT NULL ,
		[Body] nvarchar(max) NOT NULL ,
		UserID int NULL,
		Upvotes int NOT NULL default(1),
		[Status] tinyint NOT NULL default(1),
		IdeaDate datetime NOT NULL
	)

	CREATE TABLE dbo.IdeaComments (
		[CommentID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		IdeaID [int] NOT NULL ,
		[CommentDate] [datetime] NOT NULL DEFAULT (getdate()),
		[UserID] [int] NULL ,
		[Body] nvarchar(max) NOT NULL ,
		[ForTechsOnly] bit NOT NULL DEFAULT (0)
	)
END
GO

if not exists(select name from syscolumns where id=object_id('hdSettings') and name='DisableIdeas')
	alter table hdSettings add DisableIdeas bit NOT NULL DEFAULT 0
GO

if not exists (select * from syscolumns where id=object_id('Integrations') and name='GitLabEnabled')
begin
	ALTER TABLE Integrations ADD GitLabEnabled bit not null default 0
	ALTER TABLE Integrations ADD GitLabToken varchar(50) NULL
	ALTER TABLE IntegrationTicketData ADD GitLabIssueUrl varchar(300) null
end
go

if not exists(select name from syscolumns where id=object_id('hdSettings') and name='MailcheckerUnregDomains')
	alter table hdSettings add MailcheckerUnregDomains varchar(max) NULL

GO

--converting AssetLog from varchar to Nvarchar
if exists (select id from syscolumns where id=object_id('AssetLog') and name='Description' and type_name(xtype)<>'nvarchar')
	alter table AssetLog alter column Description nvarchar(500) NOT NULL
GO

if not exists(select name from syscolumns where id=object_id('hdSettings') and name='GoogleLoginDomains')
	alter table hdSettings add GoogleLoginDomains varchar(1000) NULL
GO

if not exists (select * from syscolumns where id=object_id('hdComments') and name='BouncedRecipients')
begin
	  ALTER TABLE hdComments ADD BouncedRecipients nvarchar(3000) null
end
go

--html fulltext in tickets
if not exists (select * from syscolumns where id=object_id('hdIssues') and name='FTS_filetype')
begin
	alter table [dbo].[hdIssues] add FTS_Body as 0xFFFE + CAST((Subject + N' InstanceID' + CAST(InstanceID as nvarchar(max)) + N' ' + Body) as VARBINARY(MAX))
	alter table [dbo].[hdIssues] add FTS_filetype As '.htm'
end
go

declare @installed sql_variant
select @installed=SERVERPROPERTY('IsFullTextInstalled')
IF(@installed = 1)
begin
	BEGIN TRY
		ALTER FULLTEXT INDEX ON [dbo].[hdIssues] DROP ([Body])
	END TRY
	begin catch
		print 'body not fullindexed, nevermind'
	end catch

	BEGIN TRY
		ALTER FULLTEXT INDEX ON [dbo].[hdIssues] DROP ([Subject])
	END TRY
	begin catch
		print 'subject not fullindexed, nevermind'
	end catch

	IF NOT EXISTS (select * from sys.fulltext_indexes where object_id=object_id('hdIssues'))
	BEGIN
		print 'creating fulltext index on hdIssues'
		declare @issuesConstraint sysname
		declare @issueIndexCmd varchar(1000)
		SELECT @issuesConstraint=CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME='hdIssues' AND CONSTRAINT_TYPE='PRIMARY KEY'
		SET @issueIndexCmd = 'CREATE FULLTEXT INDEX ON hdIssues  ([FTS_Body] TYPE COLUMN [FTS_filetype]) KEY INDEX ' + @issuesConstraint
		EXEC(@issueIndexCmd)
	END
	else begin
		BEGIN TRY
			ALTER FULLTEXT INDEX ON [dbo].[hdIssues] ADD ([FTS_Body] TYPE COLUMN [FTS_filetype])
		END TRY
		begin catch
			print 'fts_body already fullindexed, nevermind'
		end catch
	end
end
go

--varchar(max) for email templates
if not exists (select * from syscolumns where id=object_id('EmailTemplates') and name='EmailTemplate' and length=-1)
begin
	--drop all default constraints
	declare @sql nvarchar(max)
	set @sql = ''
	select @sql = @sql + 'alter table EmailTemplates drop constraint ' + name  + ';'
	from sys.default_constraints where parent_object_id = object_id('EmailTemplates') AND type = 'D'
	exec sp_executesql @sql

	alter table EmailTemplates alter column EmailTemplate nvarchar(MAX) NOT NULL
	alter table EmailTemplates alter column NewIssueEmailTemplate nvarchar(MAX) NOT NULL
	alter table EmailTemplates alter column WelcomeEmailBody nvarchar(MAX) NOT NULL
	alter table EmailTemplates alter column NewIssueConfirmationBody nvarchar(MAX) NOT NULL
	alter table EmailTemplates alter column ClosedTicketEmailTemplate nvarchar(MAX) NOT NULL
	alter table EmailTemplates alter column UnregUserBounceBody nvarchar(MAX) NOT NULL

	ALTER TABLE EmailTemplates ADD CONSTRAINT DF_EmailTemplate DEFAULT '' FOR EmailTemplate
	ALTER TABLE EmailTemplates ADD CONSTRAINT DF_EmailSubjectTemplate DEFAULT '' FOR EmailSubjectTemplate;
	ALTER TABLE EmailTemplates ADD CONSTRAINT DF_NewIssueEmailSubject DEFAULT '' FOR NewIssueEmailSubject;
	ALTER TABLE EmailTemplates ADD CONSTRAINT DF_NewIssueEmailTemplate DEFAULT '' FOR NewIssueEmailTemplate;
	ALTER TABLE EmailTemplates ADD CONSTRAINT DF_WelcomeEmailSubj DEFAULT '' FOR WelcomeEmailSubj;
	ALTER TABLE EmailTemplates ADD CONSTRAINT DF_WelcomeEmailBody DEFAULT '' FOR WelcomeEmailBody;
	ALTER TABLE EmailTemplates ADD CONSTRAINT DF_NewIssueConfirmationSubj DEFAULT '' FOR NewIssueConfirmationSubj;
	ALTER TABLE EmailTemplates ADD CONSTRAINT DF_NewIssueConfirmationBody DEFAULT '' FOR NewIssueConfirmationBody;
	ALTER TABLE EmailTemplates ADD CONSTRAINT DF_ClosedTicketEmailTemplate DEFAULT '' FOR ClosedTicketEmailTemplate;
	ALTER TABLE EmailTemplates ADD CONSTRAINT DF_ClosedTicketEmailSubject DEFAULT '' FOR ClosedTicketEmailSubject;
	ALTER TABLE EmailTemplates ADD CONSTRAINT DF_UnregUserBounceSubj DEFAULT '' FOR UnregUserBounceSubj;
	ALTER TABLE EmailTemplates ADD CONSTRAINT DF_UnregUserBounceBody DEFAULT '' FOR UnregUserBounceBody;
end
go

--varchar(500) for email srv passwords
if not exists (select * from syscolumns where id=object_id('hdPOPServers') and name='POPPassword' and length=300)
begin
	alter table hdPOPServers ALTER COLUMN POPPassword varchar(300) NULL
end
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SMTPPassword' and length=300)
begin
	alter table hdSettings ALTER COLUMN SMTPPassword varchar(300) NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='ExtractCompaniesFromDomains')
begin
	  ALTER TABLE hdSettings ADD ExtractCompaniesFromDomains bit NOT NULL DEFAULT 1
	  ALTER TABLE hdSettings ADD ExtractCompaniesExceptions varchar(max) NULL
end

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'OneTimeChecks') AND type in (N'U'))
begin
	CREATE TABLE dbo.OneTimeChecks (
		CheckID varchar(50) NOT NULL PRIMARY KEY
	)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='RestrictPriority')
begin
	  ALTER TABLE hdSettings ADD RestrictPriority bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='VSOCollectionName')
begin
	  ALTER TABLE Integrations ADD VSOCollectionName varchar(100) null
end
go

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='IdeasName')
begin
	  ALTER TABLE hdSettings ADD IdeasName nvarchar(100) NULL
end

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'TicketTemplates') AND type in (N'U'))
begin
	CREATE TABLE dbo.TicketTemplates (
		CategoryID int NOT NULL PRIMARY KEY,
		[Subject] nvarchar(500) not null,
		Body nvarchar(max) not null
	)
end
go

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SMTPServer' and length=100)
begin
	alter table hdSettings ALTER COLUMN SMTPServer varchar(100) NULL
end
go

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='TimesReopened')
begin
	ALTER TABLE hdIssues ADD TimesReopened int NOT NULL DEFAULT 0

	declare @sql nvarchar(max)
	set @sql = N'update hdissues set TimesReopened=cr.num
				from
				hdissues
				join (
					select count(commentid) as num, issueid from hdcomments with(nolock)
					where CommentDate>getdate()-30 and issystem=1 and (
						body like ''%The ticket has been re-opened''
						or body like ''%Das Ticket wurde wieder geöffnet''
						or body like ''%Le billet a été ré-ouvert''
						or body like ''%El tiqute ha sido reabierto''
						or body like ''%Il ticket è stato riaperto'')
					group by issueid) cr on hdissues.IssueID=cr.IssueID'
	exec sp_executesql @sql
end
go

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'KBSearchLog') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[KBSearchLog] (
		SearchID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL DEFAULT 0,
		[Date] datetime NOT NULL,
		Query nvarchar(200),
		NumResults int NOT NULL DEFAULT 0
	)

	CREATE TABLE [dbo].[KBViewLog] (
		ArticleID int NOT NULL,
		[Date] date NOT NULL,
		NumViews int NOT NULL DEFAULT 0
	)
	ALTER TABLE KBViewLog WITH NOCHECK ADD
		CONSTRAINT [PK_KBViewLog] PRIMARY KEY CLUSTERED (ArticleID, [Date])
end
go

--adding index to [hdAssets] ModelName
if not exists (select name FROM sysindexes WHERE name = 'ix_hdAssets_ModelName') begin
	CREATE INDEX ix_hdAssets_ModelName ON [dbo].[hdAssets] ([ModelName])
end
go

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='TopAnnouncementHTML')
begin
	  ALTER TABLE hdSettings ADD TopAnnouncementHTML nvarchar(max) NULL
end
go

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'IssueEmailMessageID') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[IssueEmailMessageID] (
		IssueID int NOT NULL PRIMARY KEY,
		MessageID nvarchar(1000) NOT NULL
	)
end
go

if not exists (select * from syscolumns where id=object_id('hdComments') and name='BounceReason')
begin
	  ALTER TABLE hdComments ADD BounceReason varchar(1000) NULL
end
go

--adding index to hdFileAttachments (IssueID)
if not exists (select name FROM sysindexes WHERE name = 'IX_hdFileAttachments_AssetID') begin
	CREATE INDEX IX_hdFileAttachments_AssetID ON hdFileAttachments (AssetID)
end

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'PasswordPolicies') AND type in (N'U'))
begin
	CREATE TABLE dbo.PasswordPolicies (
		[InstanceID] int NOT NULL PRIMARY KEY,
		MinLength int NOT NULL DEFAULT 8,
		Uppercase bit NOT NULL DEFAULT 1,
		Lowercase bit NOT NULL DEFAULT 1,
		Numbers bit NOT NULL DEFAULT 1,
		SpecialChars bit NOT NULL DEFAULT 0
	)
end
go

--stored comments-issues before tickets ere merged
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'MergedComments') AND type in (N'U'))
begin
	CREATE TABLE dbo.MergedComments (
		CommentID INT NOT NULL,
		IssueMergedID INT NOT NULL
	)
	ALTER TABLE MergedComments WITH NOCHECK ADD 
		CONSTRAINT [PK_MergedComments] PRIMARY KEY CLUSTERED (CommentID, IssueMergedID)
end
GO

--Idea tags
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'IdeaTags') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[IdeaTags] (
		TagID int NOT NULL,
		IdeaID int NOT NULL
	)
	ALTER TABLE IdeaTags WITH NOCHECK ADD CONSTRAINT [PK_IdeaTags] PRIMARY KEY CLUSTERED (TagID,IdeaID)
end
GO

if not exists (select * from syscolumns where id=object_id('Integrations') and name='SurveyMonkeyEnabled')
begin
	  ALTER TABLE Integrations ADD SurveyMonkeyEnabled bit NOT NULL DEFAULT 0
	  ALTER TABLE Integrations ADD SurveyMonkeyToken varchar(500) NULL
end
go

if not exists (select * from syscolumns where id=object_id('Integrations') and name='StripeEnabled')
begin
	ALTER TABLE Integrations ADD StripeEnabled bit not null default 0
	ALTER TABLE Integrations ADD StripeApiKey varchar(50) NULL
end
go

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MailcheckerRestrictedDomains')
begin
	  ALTER TABLE hdSettings ADD MailcheckerRestrictedDomains varchar(1000) NULL
end
go

if not exists (select * from syscolumns where id=object_id('hdPOPServers') and name='EwsImpersonatedUserEmail')
begin
	ALTER TABLE hdPOPServers ADD EwsImpersonatedUserEmail varchar(100) NULL
end

--creating KB subscriptions to individual articles
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'IdeaSubscribers') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.IdeaSubscribers (
		IdeaID int NOT NULL,
		UserID int NOT NULL
	)
	ALTER TABLE IdeaSubscribers WITH NOCHECK ADD
		CONSTRAINT [PK_IdeaSubscribers] PRIMARY KEY CLUSTERED (IdeaID, UserID)

	--also add existing commenters to subscribers
	declare @sql nvarchar(max)
	set @sql = N'INSERT INTO IdeaSubscribers (IdeaID, UserID)
	SELECT DISTINCT IdeaID, UserID FROM IdeaComments'
	exec sp_executesql @sql
	
END
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='CcForOutgoingEmails')
begin
	ALTER TABLE hdSettings ADD CcForOutgoingEmails bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('EmailTemplates') and name='CategoryID')
begin
	ALTER TABLE EmailTemplates ADD CategoryID int NOT NULL DEFAULT 0

	--adjust primary key
	declare @pkname sysname, @sql nvarchar(max)
	select @pkname = name FROM sys.key_constraints WHERE [type] = 'PK' AND [parent_object_id] = OBJECT_ID('EmailTemplates')
	if not @pkname is null
	begin
		set @sql = N'alter table EmailTemplates drop constraint ' + @pkname
		exec sp_executesql @sql
	end

	ALTER TABLE [dbo].[EmailTemplates] WITH NOCHECK ADD CONSTRAINT [PK_EmailTemplates] PRIMARY KEY CLUSTERED (InstanceID, CategoryID)
end
GO

if not exists (select * from syscolumns where id=object_id('hdPOPServers') and name='LastConnectResult')
begin
	ALTER TABLE hdPOPServers ADD LastConnectResult varchar(200) NULL
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='OutOfOffice')
begin
	ALTER TABLE hdUsers ADD OutOfOffice bit not null default 0
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='TicketMinChars')
begin
	ALTER TABLE hdSettings ADD TicketMinChars tinyint NOT NULL DEFAULT 0
end
GO

--new FT index
declare @installed sql_variant
select @installed=SERVERPROPERTY('IsFullTextInstalled')
IF(@installed = 1)
begin
	if exists (select * from sys.computed_columns where name='FTS_Body' and object_id=object_id('hdIssues') and definition not like '%Instance%')
	begin
		print 'recreating FT index with instance included'

		begin try
			DROP FULLTEXT INDEX ON hdIssues
		end try
		begin catch
			print 'can''t drop hdIssues FT-index, not exists, nevermind'
		end catch

		ALTER TABLE hdIssues DROP COLUMN FTS_Body

		ALTER TABLE hdIssues ADD FTS_Body as 0xFFFE + CAST((Subject + N' InstanceID' + CAST(InstanceID as nvarchar(max)) + N' ' + Body) as VARBINARY(MAX))

		declare @issuesConstraint sysname
		declare @issueIndexCmd varchar(1000)

		SELECT @issuesConstraint=CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME='hdIssues' AND CONSTRAINT_TYPE='PRIMARY KEY'

		SET @issueIndexCmd = 'CREATE FULLTEXT INDEX ON hdIssues  ([FTS_Body] TYPE COLUMN [FTS_filetype])
								KEY INDEX ' + @issuesConstraint
						
		EXEC(@issueIndexCmd)
	end
end
GO

if not exists (select * from syscolumns where id=object_id('Integrations') and name='GitHubToken')
begin
	ALTER TABLE Integrations ADD GitHubToken varchar(50) NULL
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='LastLogoutDate')
begin
	ALTER TABLE hdUsers ADD LastLogoutDate datetime null
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='StripeToken')
begin
	ALTER TABLE Integrations ADD StripeToken varchar(50) NULL
	ALTER TABLE Integrations DROP COLUMN StripeApiKey
end
GO

--subtickets
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdSubIssues') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.hdSubIssues (
		ParentID int NOT NULL,
		ChildID int  NOT NULL PRIMARY KEY
	)
	CREATE INDEX IX_hdSubIssues_ParentID ON hdSubIssues (ParentID) INCLUDE (ChildID)
END
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SamlButtonText')
begin
	ALTER TABLE hdSettings ADD SamlButtonText nvarchar(50) NULL
end
GO

--adding index for "prev tech" query which can be heavy
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_UserIDIssueIDAssignedID') begin
	CREATE INDEX IX_hdIssues_UserIDIssueIDAssignedID ON [dbo].[hdIssues] (UserID,IssueID,AssignedToUserID)
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='Rating')
begin
	ALTER TABLE hdIssues ADD Rating tinyint NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='BasecampEnabled')
begin
	ALTER TABLE Integrations ADD BasecampEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE Integrations ADD BasecampToken varchar(500) NULL
	ALTER TABLE Integrations ADD BasecampAccID varchar(20) NULL
	ALTER TABLE IntegrationTicketData ADD BasecampTodoUrl varchar(300) null
end
go

if not exists (select * from syscolumns where id=object_id('hdCustomFields') and name='ParentFieldID')
begin
	ALTER TABLE hdCustomFields ADD ParentFieldID int null
	ALTER TABLE hdCustomFieldOptions ADD ParentOptionID int null
end

--multi-category canned responses
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'CommentTemplateCategories') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[CommentTemplateCategories] (
		[TemplateID] int NOT NULL,
		[CategoryID] int NOT NULL
	)
	ALTER TABLE CommentTemplateCategories WITH NOCHECK ADD CONSTRAINT PK_CommentTemplateCategories PRIMARY KEY CLUSTERED (
		TemplateID,
		CategoryID
	)

	if exists (select * from syscolumns where id=object_id('hdCommentTemplates') and name='CategoryID')
	begin
		declare @sql nvarchar(max)
		set @sql = N'INSERT INTO CommentTemplateCategories (TemplateID, CategoryID)
		SELECT TemplateID, CategoryID FROM hdCommentTemplates WHERE CategoryID IS NOT NULL'
		exec sp_executesql @sql

		ALTER TABLE hdCommentTemplates DROP COLUMN CategoryID
	end
END
GO

if not exists (select * from syscolumns where id=object_id('Integrations') and name='MsTeamsEnabled')
begin
	ALTER TABLE Integrations ADD MsTeamsEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE Integrations ADD MsTeamsWebhookUrl varchar(500) NULL
end

if not exists (select * from syscolumns where id=object_id('UserAvatars') and name='ImageUrl')
begin
	ALTER TABLE UserAvatars ADD ImageUrl varchar(500) NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='TopAdminAnnouncementHTML')
begin
	ALTER TABLE hdSettings ADD TopAdminAnnouncementHTML nvarchar(max) NULL
	ALTER TABLE hdSettings ADD TopTechAnnouncementHTML nvarchar(max) NULL
end
go

if not exists (select * from syscolumns where id=object_id('hdStatus') and name='Color')
begin
	ALTER TABLE hdStatus ADD Color char(6) NULL
end

--signatures
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'EmailSignatures') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.EmailSignatures (
		UserID int NOT NULL PRIMARY KEY,
		EmailSignature varchar(max)
	)
END
GO

if not exists (select * from syscolumns where id=object_id('hdCategories') and name='Disabled')
begin
	ALTER TABLE hdCategories ADD Disabled bit not null default 0
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='GitLabBaseUrl')
begin
	ALTER TABLE Integrations ADD GitLabBaseUrl varchar(100) null
end

if not exists (select * from syscolumns where id=object_id('hdPOPServers') and name='Office365Token')
begin
	ALTER TABLE hdPOPServers ADD Office365Token varchar(4000)
	ALTER TABLE hdPOPServers ADD Office365RefreshToken varchar(4000)
end

--add CompanyID index to hdUsers b/c we're using FORCESEEK
if not exists (select name FROM sysindexes WHERE name = 'IX_hdUsers_CompanyID') begin
	CREATE INDEX IX_hdUsers_CompanyID ON hdUsers (CompanyID)
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='DarkMode')
begin
	ALTER TABLE hdUsers ADD DarkMode bit not null default 0
end
go

if not exists (select * from syscolumns where id=object_id('Integrations') and name='OneDriveEnabled')
begin
	ALTER TABLE Integrations ADD OneDriveEnabled bit NOT NULL DEFAULT 0
end

--multi-category canned responses
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'CustomHolidays') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[CustomHolidays] (
		[InstanceID] int NOT NULL,
		[Holiday] date NOT NULL
	)
	ALTER TABLE [dbo].[CustomHolidays] WITH NOCHECK ADD CONSTRAINT PK_CustomHolidays PRIMARY KEY CLUSTERED (
		[InstanceID], [Holiday]
	)
END
GO

--adding some indexes
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssueTags_IssueID') begin
	CREATE INDEX IX_hdIssueTags_IssueID ON [dbo].[hdIssueTags] ([IssueID]) INCLUDE ([TagID])
end
if not exists (select name FROM sysindexes WHERE name = 'IX_InstanceID_UsageType') begin
	CREATE INDEX IX_InstanceID_UsageType ON [dbo].[hdCustomFields] ([InstanceID],[UsageType])
end
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_InstanceStatusAssigned_IncludeStatsFields') begin
	CREATE INDEX IX_hdIssues_InstanceStatusAssigned_IncludeStatsFields ON [dbo].[hdIssues] ([InstanceID],[AssignedToUserID],[StatusID])
		INCLUDE ([IssueID],[UserID],[CategoryID],[UpdatedByUser],[UpdatedForTechView],[IssueDate],[ResolvedDate],[Subject],[UpdatedByPerformer],[Priority],[LastUpdated],[Origin],[DueDate])
end
if not exists (select name FROM sysindexes WHERE name = 'ix_hdCustFIdVls_IssueID_incValue') begin
	CREATE INDEX ix_hdCustFIdVls_IssueID_incValue ON [dbo].[hdCustomFieldValues] ([IssueID]) INCLUDE ([Value],[FieldID])
end
go

if not exists (select * from syscolumns where id=object_id('Integrations') and name='BoxEnabled')
begin
	ALTER TABLE Integrations ADD BoxEnabled bit NOT NULL DEFAULT 0
end
if exists(select name from syscolumns where id=object_id('hdFileAttachments') and name='GoogleDriveUrl' and length<2000)--enlarge googledriveurl field
begin
	alter table hdFileAttachments alter column GoogleDriveUrl varchar(2000) NULL
end
go

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdCustomFieldTextValues') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].hdCustomFieldTextValues (
		IssueID int NOT NULL PRIMARY KEY,
		CustomFieldsText nvarchar(max)
	)

	insert into hdCustomFieldTextValues(IssueID, CustomFieldsText)
	select IssueID, N'InstanceID' + CAST(hdCustomFields.InstanceID as nvarchar(max)) + (select ' ' + cast([Value] as nvarchar(max)) from hdCustomFieldValues WHERE IssueID = cfv.IssueID FOR XML PATH(''))
	from hdCustomFieldValues cfv
	join hdCustomFields on cfv.FieldID = hdCustomFields.FieldID
	where hdCustomFields.Type in (1,3,5)
	group by IssueID, hdCustomFields.InstanceID
END
GO

IF NOT EXISTS (select * from sys.objects where type = 'TR' and name = 'updCfText')	
	exec sp_executesql N'
	CREATE TRIGGER updCfText ON dbo.hdCustomFieldValues AFTER DELETE, UPDATE, INSERT
	as begin
		--exit if non-text fields are updated
		if not exists (select 1 from inserted join hdCustomFields cf on cf.FieldID=inserted.FieldID where cf.Type in (1,3,5))
			return;

		--delete removed
		delete from hdCustomFieldTextValues where issueid in (select issueid from deleted union all select issueid from inserted)

		--insert (rewrite the data)
		BEGIN TRY --JFDI-pattern
			insert into hdCustomFieldTextValues(IssueID, CustomFieldsText)
			select IssueID, N''InstanceID'' + CAST(hdCustomFields.InstanceID as nvarchar(max)) + (select '' '' + cast([Value] as nvarchar(max)) from hdCustomFieldValues WITH(NOLOCK) WHERE IssueID = cfv.IssueID FOR XML PATH(''''))
			from hdCustomFieldValues cfv WITH(NOLOCK)
			join hdCustomFields on cfv.FieldID = hdCustomFields.FieldID
			where hdCustomFields.Type in (1,3,5) and IssueID in (SELECT IssueID from inserted)
			group by IssueID, hdCustomFields.InstanceID
		END TRY
		BEGIN CATCH
			IF ERROR_NUMBER() <> 2627 AND ERROR_NUMBER() <> 2601
				THROW 
		END CATCH
	end'
GO

declare @installed sql_variant
select @installed=SERVERPROPERTY('IsFullTextInstalled')
IF @installed = 1 AND NOT EXISTS (select * from sys.fulltext_index_columns fic join sys.columns c on c.[object_id] = fic.[object_id] and c.[column_id] = fic.[column_id] where fic.object_id=object_id('hdCustomFieldTextValues') and name='CustomFieldsText')
begin
	--creating CF index
	declare @cfConstraint sysname
	declare @cfIndexCmd nvarchar(1000)

	SELECT @cfConstraint=CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME='hdCustomFieldTextValues' AND CONSTRAINT_TYPE='PRIMARY KEY'

	set @cfIndexCmd = N'CREATE FULLTEXT INDEX ON hdCustomFieldTextValues (CustomFieldsText) KEY INDEX ' + @cfConstraint
	exec sp_executesql @cfIndexCmd
END
GO

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='LastUpdatedByUserID')
begin
	ALTER TABLE hdIssues ADD LastUpdatedByUserID int NULL
	--add column to index
	if not exists (select c.name, i.name, i.type_desc
		from sys.indexes i
			join sys.index_columns ic on i.index_id = ic.index_id and i.object_id = ic.object_id
			join sys.columns c on ic.column_id = c.column_id and ic.object_id = c.object_id
		where ic.is_included_column = 1 and i.name='IX_hdIssues_InstanceStatusAssigned_IncludeStatsFields' and c.name = 'LastUpdatedByUserID')
	begin
		DROP INDEX IX_hdIssues_InstanceStatusAssigned_IncludeStatsFields ON hdIssues --drop and recreate
		CREATE INDEX IX_hdIssues_InstanceStatusAssigned_IncludeStatsFields ON [dbo].[hdIssues] ([InstanceID],[AssignedToUserID],[StatusID])
			INCLUDE ([IssueID],[UserID],[CategoryID],[UpdatedByUser],[UpdatedForTechView],[IssueDate],[ResolvedDate],[Subject],[UpdatedByPerformer],[Priority],[LastUpdated],[Origin],[DueDate],LastUpdatedByUserID)
	end
end
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdCustomFieldValuesForUsers') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdCustomFieldValuesForUsers] (
		[UserID] [int] NOT NULL,
		[FieldID] [int] NOT NULL,
		[Value] sql_variant NOT NULL,
		[OptionID] int NULL
	)
	ALTER TABLE [dbo].[hdCustomFieldValuesForUsers] WITH NOCHECK ADD 
		CONSTRAINT [PK_hdCustomFieldValuesForUsers] PRIMARY KEY CLUSTERED 
		(
			[UserID],
			[FieldID]
		)
END
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SamlAutoRedirect')
begin
	ALTER TABLE hdSettings ADD SamlAutoRedirect bit NOT NULL DEFAULT 0
end
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdCustomFieldValuesForCompanies') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdCustomFieldValuesForCompanies] (
		[CompanyID] [int] NOT NULL,
		[FieldID] [int] NOT NULL,
		[Value] sql_variant NOT NULL,
		[OptionID] int NULL
	)

	ALTER TABLE [dbo].[hdCustomFieldValuesForCompanies] WITH NOCHECK ADD 
		CONSTRAINT [PK_hdCustomFieldValuesForCompanies] PRIMARY KEY CLUSTERED 
		(
			[CompanyID],
			[FieldID]
		)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdUserLogonHistory') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdUserLogonHistory] (
		UserID int NOT NULL,
		LogonDate datetime NOT NULL,
		IPAddress varchar(50) NOT NULL
	)
	ALTER TABLE [dbo].[hdUserLogonHistory] WITH NOCHECK ADD
		CONSTRAINT [PK_hdUserLogonHistory] PRIMARY KEY CLUSTERED (UserID, LogonDate)
END
GO

if not exists (select * from syscolumns where id=object_id('Ideas') and name='UpvoteUserIds')
begin
	ALTER TABLE Ideas ADD UpvoteUserIds varchar(1000) null, UpvoteIpHashes varchar(1000) null
end
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdSharedIssues') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdSharedIssues] (IssueID int NOT NULL PRIMARY KEY)
END
GO

if not exists (select * from syscolumns where id=object_id('EmailSignatures') and name='MsgCount')
begin
	ALTER TABLE EmailSignatures ADD MsgCount tinyint NOT NULL default 0
end

if exists(select name from syscolumns where id=object_id('hdPopServers') and name='Office365Token' and length<4000)
begin
	ALTER TABLE hdPopServers ALTER COLUMN Office365Token varchar(4000) null
end
if exists(select name from syscolumns where id=object_id('hdPopServers') and name='Office365RefreshToken' and length<4000)
begin
	ALTER TABLE hdPopServers ALTER COLUMN Office365RefreshToken varchar(4000) null
end

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'SecondaryEmails') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.SecondaryEmails(
		UserID int NOT NULL,
		Email varchar(200) NOT NULL
	)
	ALTER TABLE SecondaryEmails WITH NOCHECK ADD
		CONSTRAINT [PK_SecondaryEmails] PRIMARY KEY CLUSTERED (UserID, Email)
END
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='ReportPermissionsJson')
begin
	  ALTER TABLE hdSettings ADD ReportPermissionsJson nvarchar(500) NULL
end

if exists(select name from syscolumns where id=object_id('hdSettings') and name='SamlCertificate' and length<4500)
begin
	ALTER TABLE hdSettings ALTER COLUMN SamlCertificate varchar(4500) null
end

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'AutomationExecutionLog') AND type in (N'U'))
BEGIN
	CREATE TABLE dbo.AutomationExecutionLog (
		RuleID int NOT NULL,
		CommentID int NOT NULL
	)
	ALTER TABLE AutomationExecutionLog WITH NOCHECK ADD 
		CONSTRAINT [PK_AutomationExecutionLog] PRIMARY KEY CLUSTERED (RuleID, CommentID)

	CREATE INDEX IX_AutomationExecutionLog_RuleID ON AutomationExecutionLog (RuleID)
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'CommentTemplateGroups') AND type in (N'U'))
BEGIN
	ALTER TABLE hdCommentTemplates ADD GroupID int NULL

	CREATE TABLE [dbo].[CommentTemplateGroups] (
		GroupID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		[Name] nvarchar(100) NOT NULL
	)
END
GO

--Asset tags
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'AssetTags') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[AssetTags] (
		TagID int NOT NULL,
		AssetID int NOT NULL
	)
	ALTER TABLE AssetTags WITH NOCHECK ADD CONSTRAINT [PK_AssetTags] PRIMARY KEY CLUSTERED (TagID,AssetID)
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableRememberMe')
begin
	ALTER TABLE hdSettings ADD DisableRememberMe bit NOT NULL DEFAULT 0
end
GO

if not exists (select * from syscolumns where id=object_id('Integrations') and name='AsanaRefreshToken')
begin
	ALTER TABLE Integrations ADD AsanaRefreshToken varchar(300) null
end

--enlarge the password field so enxrypted strings fits there
if exists(select name from syscolumns where id=object_id('hdSettings') and name='SMTPPassword' and length<500)
begin
	alter table hdSettings alter column SMTPPassword [varchar](500) NULL
end

--we made a release with a bad upgrade script that made this column NOT NULL. The below script reverses the change
DECLARE @nullable INT;
SELECT @nullable = COLUMNPROPERTY(OBJECT_ID('hdSettings', 'U'), 'SMTPPassword', 'AllowsNull')
IF (@nullable = 0)
BEGIN
	alter table hdSettings alter column SMTPPassword [varchar](500) NULL
END
GO

--adding InstanceID to comments for better full text search
if not exists (select * from syscolumns where id=object_id('hdComments') and name='InstanceID')
begin
	alter table hdComments ADD InstanceID int null
	exec sp_executesql N'
	alter table hdComments add FTS_Body as
		CASE WHEN IsSystem=1 THEN CAST(NULL as VARBINARY(MAX))
		ELSE 0xFFFE + CAST((N''InstanceID'' + CAST(InstanceID as nvarchar(max)) + N'' '' + Body) as VARBINARY(MAX))
		END'
	alter table hdComments add FTS_filetype As '.htm'

	exec sp_executesql N'
		UPDATE hdComments SET InstanceID = hdIssues.InstanceID
		FROM hdComments JOIN hdIssues ON hdComments.IssueID = hdIssues.IssueID
		WHERE IsSystem=0 AND hdComments.InstanceID IS NULL --AND CommentID>200000000 --for hosted version'
end
GO

declare @installed sql_variant
select @installed=SERVERPROPERTY('IsFullTextInstalled')
IF @installed = 1 AND NOT EXISTS (select * from sys.fulltext_index_columns fic join sys.columns c on c.[object_id] = fic.[object_id] and c.[column_id] = fic.[column_id] where fic.object_id=object_id('hdComments') and name='FTS_Body')
begin
	--drop old "Body" index on comments just in case
	begin try
		DROP FULLTEXT INDEX ON dbo.hdComments
	end try begin catch end catch
	
	declare @constraint sysname

	SELECT @constraint=CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME='hdComments' AND CONSTRAINT_TYPE='PRIMARY KEY'
	EXEC('CREATE FULLTEXT INDEX ON hdComments ([FTS_Body] TYPE COLUMN [FTS_filetype]) KEY INDEX ' + @constraint) --+ ' ON (hdFulltextCatalog, FILEGROUP SECONDARY)'--hosted

	--dropping old stuff
	begin try
		DROP FULLTEXT INDEX ON dbo.vwNonSystemComments
	end try begin catch end catch
	begin try
		DROP VIEW dbo.vwNonSystemComments
	end try begin catch end catch
end
GO

if not exists (select name FROM sysindexes WHERE name = 'IX_hdUsers_InstanceIDUsernameDisabled')
	CREATE INDEX IX_hdUsers_InstanceIDUsernameDisabled ON hdUsers (InstanceID, Username, [Disabled])
if not exists (select name FROM sysindexes WHERE name = 'IX_hdUsers_InstanceIDEmailDisabled')
	CREATE INDEX IX_hdUsers_InstanceIDEmailDisabled ON hdUsers (InstanceID, Email, [Disabled])

if exists (select name FROM sysindexes WHERE name = 'IX_hdUsers_InstanceIDPasswordDisabled')
	DROP INDEX IX_hdUsers_InstanceIDPasswordDisabled ON hdUsers

--enlarge the password field so enxrypted strings fits there
if exists(select name from syscolumns where id=object_id('hdSections') and name='Name' and length<250)
begin
	alter table hdSections alter column [Name] nvarchar (250) NOT NULL
end
GO

--remove dropboxurl (obsolete)
if exists(select name from syscolumns where id=object_id('hdFileAttachments') and name='DropboxUrl')
begin
	EXEC('UPDATE hdFileAttachments SET GoogleDriveUrl=DropboxUrl WHERE DropboxUrl IS NOT NULL AND DropboxUrl<>''''')
	EXEC('ALTER TABLE hdFileAttachments DROP COLUMN DropboxUrl')
end
GO

--enlarge the assetlog field so enxrypted strings fits there
if exists(select name from syscolumns where id=object_id('AssetLog') and name='[Description]' and length<501)
begin
	alter table [AssetLog] alter column [Description]  NVARCHAR (max) NOT NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='RestrictAdminAssignment')
begin
	ALTER TABLE hdSettings ADD RestrictAdminAssignment bit NOT NULL DEFAULT 0
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableDefaultHolidays')
begin
	ALTER TABLE hdSettings ADD DisableDefaultHolidays bit NOT NULL DEFAULT 0
end
GO

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='IsDepartmentManager')
begin
	ALTER TABLE hdUsers ADD IsDepartmentManager bit not null DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='TeamViewerEnabled')
begin
	ALTER TABLE Integrations ADD TeamViewerEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE Integrations ADD TeamViewerFieldID int NULL
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SameDepartmentUserPermission')
begin
	ALTER TABLE hdSettings ADD SameDepartmentUserPermission bit not null default 0
end

--update trigger
if not exists (select * from sys.objects where type = 'TR' and name = 'updCfText' and OBJECT_DEFINITION(object_id) LIKE '%non-text%')
begin
	exec sp_executesql N'
	ALTER TRIGGER updCfText ON dbo.hdCustomFieldValues AFTER DELETE, UPDATE, INSERT
	as begin
		--exit if non-text fields are updated
		if not exists (select 1 from inserted join hdCustomFields cf on cf.FieldID=inserted.FieldID where cf.Type in (1,3,5))
			return;

		--delete removed
		delete from hdCustomFieldTextValues where issueid in (select issueid from deleted union all select issueid from inserted)

		--insert (rewrite the data)
		BEGIN TRY --JFDI-pattern
			insert into hdCustomFieldTextValues(IssueID, CustomFieldsText)
			select IssueID, N''InstanceID'' + CAST(hdCustomFields.InstanceID as nvarchar(max)) + (select '' '' + cast([Value] as nvarchar(max)) from hdCustomFieldValues WITH(NOLOCK) WHERE IssueID = cfv.IssueID FOR XML PATH(''''))
			from hdCustomFieldValues cfv WITH(NOLOCK)
			join hdCustomFields on cfv.FieldID = hdCustomFields.FieldID
			where hdCustomFields.Type in (1,3,5) and IssueID in (SELECT IssueID from inserted)
			group by IssueID, hdCustomFields.InstanceID
		END TRY
		BEGIN CATCH
			IF ERROR_NUMBER() <> 2627 AND ERROR_NUMBER() <> 2601
			  THROW 
		END CATCH
	end'
end

--converting tickettemplates from varchar to Nvarchar
if exists (select id from syscolumns where id=object_id('TicketTemplates') and name='Subject' and type_name(xtype)<>'nvarchar')
	alter table TicketTemplates alter column Subject nvarchar(500) NOT NULL
if exists (select id from syscolumns where id=object_id('TicketTemplates') and name='Body' and type_name(xtype)<>'nvarchar')
	alter table TicketTemplates alter column Body nvarchar(max) NOT NULL

--ticket purging
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DaysToDeleteAfter')
begin
	ALTER TABLE hdSettings ADD DaysToDeleteAfter int not null default 0
end

--varchar(max) for MailcheckerUnregDomains
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MailcheckerUnregDomains' and length=-1)
begin
	alter table hdSettings alter column MailcheckerUnregDomains nvarchar(max) NULL
end

--allowUserRegistration column
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='TechOnlyEmailReplies')
begin
	  ALTER TABLE hdSettings ADD TechOnlyEmailReplies bit NOT NULL DEFAULT 0
end

--HideSystemComments column
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='HideSystemComments')
begin
	  ALTER TABLE hdSettings ADD HideSystemComments bit not null default 0
end

--HideSystemComments column
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='ThankYouText')
begin
	  ALTER TABLE hdSettings ADD ThankYouText nvarchar(max) null
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SMTPOAuthToken')
begin
	ALTER TABLE hdSettings ADD SMTPOAuthToken varchar(4000) null, SMTPOAuthRefreshToken varchar(4000) null
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='RequireMFA')
begin
	ALTER TABLE hdSettings ADD RequireMFA bit not null default 0
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='ClickUpEnabled')
begin
	ALTER TABLE Integrations ADD ClickUpEnabled bit NOT NULL DEFAULT 0, ClieckUpToken varchar(80) NULL
	ALTER TABLE IntegrationTicketData ADD ClickUpTaskUrl varchar(300) null
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='WhatsAppEnabled')
begin
	ALTER TABLE Integrations ADD WhatsAppEnabled bit NOT NULL DEFAULT 0, WhatsAppPhoneNumberId varchar(50) NULL, WhatsAppToken varchar(500) null
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='SnipeItEnabled')
begin
	ALTER TABLE Integrations ADD SnipeItEnabled bit NOT NULL DEFAULT 0, SnipeItUrl varchar(100) NULL, SnipeItToken varchar(1500) null
	ALTER TABLE IntegrationTicketData ADD SnipeItIds varchar(1500) null
end
GO

--saving filters to db
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'UserSavedFilters') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[UserSavedFilters] (
		UserID int NOT NULL PRIMARY KEY,
		SavedFiltersJson varchar(max) NULL
	)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='UselessFilterWhitelist')
begin
	ALTER TABLE hdSettings ADD UselessFilterWhitelist varchar(max) null
end

--saving SID from AD
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'UserADSids') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[UserADSids] (
		UserID int NOT NULL PRIMARY KEY,
		UserSID varchar(100) NOT NULL
	)
end

--update trigger
if not exists (select * from sys.objects where type = 'TR' and name = 'updCfText' and OBJECT_DEFINITION(object_id) LIKE '%2627%')
begin
	exec sp_executesql N'
	ALTER TRIGGER updCfText ON dbo.hdCustomFieldValues AFTER DELETE, UPDATE, INSERT
	as begin
		--exit if non-text fields are updated
		if not exists (select 1 from inserted join hdCustomFields cf on cf.FieldID=inserted.FieldID where cf.Type in (1,3,5))
			return;

		--delete removed
		delete from hdCustomFieldTextValues where issueid in (select issueid from deleted union all select issueid from inserted)

		--insert (rewrite the data)
		BEGIN TRY --JFDI-pattern
			insert into hdCustomFieldTextValues(IssueID, CustomFieldsText)
			select IssueID, N''InstanceID'' + CAST(hdCustomFields.InstanceID as nvarchar(max)) + (select '' '' + cast([Value] as nvarchar(max)) from hdCustomFieldValues WITH(NOLOCK) WHERE IssueID = cfv.IssueID FOR XML PATH(''''))
			from hdCustomFieldValues cfv WITH(NOLOCK)
			join hdCustomFields on cfv.FieldID = hdCustomFields.FieldID
			where hdCustomFields.Type in (1,3,5) and IssueID in (SELECT IssueID from inserted)
			group by IssueID, hdCustomFields.InstanceID
		END TRY
		BEGIN CATCH
			IF ERROR_NUMBER() <> 2627 AND ERROR_NUMBER() <> 2601
			  THROW 
		END CATCH
	end'
end


--logging ticket overdue
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'OverdueLog') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[OverdueLog] (
		IssueID int NOT NULL PRIMARY KEY,
		TimesOverDue int NOT NULL
	)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AppURL')
begin
	ALTER TABLE hdSettings ADD AppURL varchar(200) null
end

if not exists (select * from syscolumns where id=object_id('Integrations') and name='GPTEnabled')
begin
	ALTER TABLE Integrations ADD GPTEnabled bit NOT NULL DEFAULT 0
	ALTER TABLE Integrations ADD GPTApiKey varchar(100) NULL
end

--enlarge JIra password (they give longer tokens now)
if exists(select name from syscolumns where id=object_id('Integrations') and name='JiraPassword' and length<500)
begin
	alter table Integrations alter column JiraPassword [varchar](500) NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='EmailOnTakeoverIncludeAdmins')
begin
	ALTER TABLE hdSettings ADD EmailOnTakeoverIncludeAdmins bit not null default 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='Office365GccClientId')
begin
	ALTER TABLE hdSettings ADD Office365GccClientId varchar(200) null
	ALTER TABLE hdSettings ADD Office365GccClientSecret varchar(200) null
end

if not exists (select * from syscolumns where id=object_id('hdStatus') and name='OrderByNumber')
begin
	ALTER TABLE hdStatus ADD OrderByNumber int NOT NULL DEFAULT 1
end
GO

--custom priorities
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'Priorities') AND type in (N'U'))
begin
	CREATE TABLE dbo.Priorities (
		[PriorityID] int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NULL,
		[Name] nvarchar (50) NOT NULL
	)

	declare @sql nvarchar(max)
	set @sql = N'SET IDENTITY_INSERT Priorities ON
	INSERT INTO Priorities (PriorityID, Name) VALUES (-1, ''Low'')
	INSERT INTO Priorities (PriorityID, Name) VALUES (0, ''Normal'')
	INSERT INTO Priorities (PriorityID, Name) VALUES (1, ''High'')
	INSERT INTO Priorities (PriorityID, Name) VALUES (2, ''Critical'')
	SET IDENTITY_INSERT Priorities OFF'
	exec sp_executesql @sql
end
GO

--converting KBTitle from varchar to Nvarchar
if exists (select id from syscolumns where id=object_id('hdSettings') and name='KBTitle' and type_name(xtype)<>'nvarchar')
	alter table hdSettings alter column KBTitle nvarchar(max) NULL
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableAutoTimerPause')
begin
	ALTER TABLE hdSettings ADD DisableAutoTimerPause bit not null default 0
	ALTER TABLE hdSettings ADD AutoTimerOnlyWhenAssigned bit not null default 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AllowFreeEmails')
begin
	ALTER TABLE hdSettings ADD AllowFreeEmails bit not null default 1
end

if not exists (select * from syscolumns where id=object_id('KBArticles') and name='GPTIndexed')
	ALTER TABLE KBArticles add GPTIndexed bit not null default(0)

if not exists (select *
from syscolumns
where id=object_id('Integrations') and name='GPTIncludeKB')
begin
	ALTER TABLE Integrations ADD GPTIncludeKB bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='NewTicketLayout')
begin
	ALTER TABLE hdSettings ADD NewTicketLayout varbinary(50) null
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='DateCreated')
begin
	ALTER TABLE hdUsers ADD DateCreated datetime NULL DEFAULT(getdate())
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='RestrictAssetsToAdmins')
begin
	ALTER TABLE hdSettings ADD RestrictAssetsToAdmins bit not null default 0
end
go

if not exists (select * from syscolumns where id=object_id('Integrations') and name='LansweeperEnabled')
begin
	ALTER TABLE Integrations ADD LansweeperEnabled bit NOT NULL default 0, LansweeperToken varchar(200), LansweeperSiteId varchar(100)
	
	ALTER TABLE IntegrationTicketData ADD LansweeperIds varchar(1000) null
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='IPRestrictions')
begin
	ALTER TABLE hdSettings ADD IPRestrictions varchar(max) NULL
end
--tail cut for CD--